# GraphQL Architect

This is a Neo4j Graph App for building GraphQL APIs backed by Neo4j.

## Current stack

**[Create React App Bootstrapper](https://reactjs.org/docs/create-a-new-react-app.html#create-react-app)** - built with [typescript](https://create-react-app.dev/docs/adding-typescript/) for typechecking, [eslint and prettier](https://prettier.io/docs/en/integrating-with-linters.html) for formatting and [Github actions](https://github.com/features/actions) for CI

**[Semantic React UI](https://react.semantic-ui.com/)** - semantic UI react component library

**[Fomantic UI](https://fomantic-ui.com/)** - A community fork of semantic UI, that supplies the built stylesheets

**[React Testing Library](https://testing-library.com/docs/react-testing-library/intro)** - A testing library for react, look at any `*.test.tsx` file for reference, that aren't too many atm 😸

## Setup

1. Clone the repo down and then run `npm install`.
2. Install public modules and dedupe, then build and pack with `npm run first:install`
3. Start it up - `npm start` or follow the steps to add to [desktop](#neo4j-desktop)

## Development

`npm run start` will run the HMR that is part of `react-scripts` and allow iteration on actions and styles that are not attached to the neo4j desktop build.

### Common Custom Dev Scripts

- **dev** - runs tests and HMR in parallel
- **pack:dev** - removes build files and packs the tarball without running build on theming
- **pack:clean** - removes build files and packs the tarball, runs build on theming
- **theme:build** - builds files for css theme
- **theme:watch** - watchs fomantic files in /assets and rebuilds

### Theme Dev

If you are planning on doing theme development you can use:

```bash
$ cd src/assets/fomantic && npx gulp watch
or
$ npm run theme:watch
```

This process is fairly time intensive.

### Neo4j Desktop Dev <a name="neo4j-desktop"></a>

To build for Neo4j Desktop you will need to build everything into the build folder and then zip up the corresponding files and drop them into the installer pane on neo4j Desktop.

1. `npm run pack:dev` - This command may fail the first time you are running in, in which case you can use `npm run build` and then `npm pack`
2. Once everything is packed start up the current development build of Neo4j desktop (you might need to ask around for this one), it uses port 3000 so make sure to shut down `react-scripts`
3. Turn on development mode in Neo4j desktop
   ![Focused Png of Neo4j Desktop dev tools][desktop-dev-tools]

4. Establish a new graph to work with in the neo4j browser
   ![Create new graph][browser-window]

5. Start your graph

   ![Starting graph][graph-starting]

6. Drag the tarball into the install panel and open the app

   ![Install App][install-focus]

## Production

### Desktop

For Neo4j Desktop the same packaging process can be used

### Web

While still working with limited functionality, the current production url is here: https://gallant-northcutt-5ebc06.netlify.com

This app is hosted on netlify, check with [Will Lyons](https://github.com/johnymontana) for team access

[desktop-dev-tools]: /docs/images/dev-tools-desktop.png 'Development mode for Neo4j Desktop'
[browser-window]: /docs/images/browser-window.png 'Create new graph in browser'
[graph-starting]: /docs/images/graph-starting.png 'Starting Neo4j graph'
[install-focus]: /docs/images/install-focus.png 'Install area for tarball'
