"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs2/helpers/interopRequireDefault");

var _Object$defineProperty2 = require("@babel/runtime-corejs2/core-js/object/define-property");

_Object$defineProperty2(exports, "__esModule", {
  value: true
});

exports.buildNodeOutputFields = exports.buildRelationshipFilters = exports.augmentRelationshipQueryAPI = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/define-property"));

var _defineProperties = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/define-properties"));

var _getOwnPropertyDescriptors = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/get-own-property-descriptors"));

var _getOwnPropertyDescriptor = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/get-own-property-descriptor"));

var _getOwnPropertySymbols = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/get-own-property-symbols"));

var _keys = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/keys"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/toConsumableArray"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/slicedToArray"));

var _defineProperty3 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/defineProperty"));

var _relationship = require("./relationship");

var _augment = require("../../augment");

var _types = require("../../types/types");

var _fields = require("../../fields");

var _inputValues = require("../../input-values");

var _directives = require("../../directives");

var _ast = require("../../ast");

var _federation = require("../../../federation");

function ownKeys(object, enumerableOnly) { var keys = (0, _keys["default"])(object); if (_getOwnPropertySymbols["default"]) { var symbols = (0, _getOwnPropertySymbols["default"])(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return (0, _getOwnPropertyDescriptor["default"])(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty3["default"])(target, key, source[key]); }); } else if (_getOwnPropertyDescriptors["default"]) { (0, _defineProperties["default"])(target, (0, _getOwnPropertyDescriptors["default"])(source)); } else { ownKeys(Object(source)).forEach(function (key) { (0, _defineProperty2["default"])(target, key, (0, _getOwnPropertyDescriptor["default"])(source, key)); }); } } return target; }

/**
 * An enum describing which arguments are implemented for
 * relationship type fields in the Query API
 */
var RelationshipQueryArgument = _objectSpread({}, _inputValues.PagingArgument, {}, _inputValues.OrderingArgument, {}, _inputValues.FilteringArgument);
/**
 * Given the results of augmentRelationshipTypeFields, builds or
 * augments the AST definition of the Query operation field and
 * any generated input or output types required for translation
 */


var augmentRelationshipQueryAPI = function augmentRelationshipQueryAPI(_ref) {
  var typeName = _ref.typeName,
      definition = _ref.definition,
      fieldArguments = _ref.fieldArguments,
      fieldName = _ref.fieldName,
      outputType = _ref.outputType,
      fromType = _ref.fromType,
      toType = _ref.toType,
      typeDefinitionMap = _ref.typeDefinitionMap,
      typeExtensionDefinitionMap = _ref.typeExtensionDefinitionMap,
      generatedTypeMap = _ref.generatedTypeMap,
      nodeInputTypeMap = _ref.nodeInputTypeMap,
      relationshipInputTypeMap = _ref.relationshipInputTypeMap,
      outputTypeWrappers = _ref.outputTypeWrappers,
      config = _ref.config,
      relationshipName = _ref.relationshipName,
      fieldType = _ref.fieldType,
      propertyOutputFields = _ref.propertyOutputFields;

  var queryTypeNameLower = _types.OperationType.QUERY.toLowerCase();

  if ((0, _augment.shouldAugmentRelationshipField)(config, queryTypeNameLower, fromType, toType)) {
    var _getTypeDefiningField = getTypeDefiningField({
      typeName: typeName,
      definition: definition,
      fieldName: fieldName,
      typeDefinitionMap: typeDefinitionMap,
      typeExtensionDefinitionMap: typeExtensionDefinitionMap
    }),
        _getTypeDefiningField2 = (0, _slicedToArray2["default"])(_getTypeDefiningField, 2),
        definingType = _getTypeDefiningField2[0],
        isImplementedField = _getTypeDefiningField2[1];

    if (isImplementedField) typeName = definingType;
    var relatedType = decideRelatedType({
      typeName: typeName,
      definition: definition,
      fromType: fromType,
      toType: toType
    });

    var _transformRelationshi = transformRelationshipTypeFieldOutput({
      typeName: typeName,
      relatedType: relatedType,
      fieldArguments: fieldArguments,
      fieldName: fieldName,
      outputType: outputType,
      fromType: fromType,
      toType: toType,
      typeDefinitionMap: typeDefinitionMap,
      generatedTypeMap: generatedTypeMap,
      outputTypeWrappers: outputTypeWrappers,
      config: config,
      relationshipName: relationshipName,
      fieldType: fieldType,
      propertyOutputFields: propertyOutputFields
    });

    var _transformRelationshi2 = (0, _slicedToArray2["default"])(_transformRelationshi, 2);

    fieldType = _transformRelationshi2[0];
    generatedTypeMap = _transformRelationshi2[1];

    var _augmentRelationshipT = augmentRelationshipTypeFieldInput({
      typeName: typeName,
      relatedType: relatedType,
      fieldArguments: fieldArguments,
      fieldName: fieldName,
      isImplementedField: isImplementedField,
      outputType: outputType,
      fromType: fromType,
      toType: toType,
      typeDefinitionMap: typeDefinitionMap,
      typeExtensionDefinitionMap: typeExtensionDefinitionMap,
      generatedTypeMap: generatedTypeMap,
      nodeInputTypeMap: nodeInputTypeMap,
      relationshipInputTypeMap: relationshipInputTypeMap,
      outputTypeWrappers: outputTypeWrappers,
      config: config
    });

    var _augmentRelationshipT2 = (0, _slicedToArray2["default"])(_augmentRelationshipT, 3);

    fieldArguments = _augmentRelationshipT2[0];
    generatedTypeMap = _augmentRelationshipT2[1];
    nodeInputTypeMap = _augmentRelationshipT2[2];
  }

  return [fieldType, fieldArguments, typeDefinitionMap, generatedTypeMap, nodeInputTypeMap];
};

exports.augmentRelationshipQueryAPI = augmentRelationshipQueryAPI;

var getTypeDefiningField = function getTypeDefiningField(_ref2) {
  var typeName = _ref2.typeName,
      definition = _ref2.definition,
      fieldName = _ref2.fieldName,
      typeDefinitionMap = _ref2.typeDefinitionMap,
      typeExtensionDefinitionMap = _ref2.typeExtensionDefinitionMap;
  var definitionInterfaces = definition.interfaces || [];
  var interfaces = (0, _toConsumableArray2["default"])(definitionInterfaces);
  var typeExtensions = typeExtensionDefinitionMap[typeName] || [];
  typeExtensions.forEach(function (extension) {
    var extendedImplementations = extension.interfaces;

    if (extendedImplementations && extendedImplementations.length) {
      interfaces.push.apply(interfaces, (0, _toConsumableArray2["default"])(extendedImplementations));
    }
  });
  var definingType = typeName; // field is defined by interface implemented by this type

  var isImplementedField = false;

  if (interfaces && interfaces.length) {
    interfaces.forEach(function (namedType) {
      var unwrappedType = (0, _fields.unwrapNamedType)({
        type: namedType
      });
      var interfaceName = unwrappedType.name;
      var interfaceTypes = [];
      var typeDefinition = typeDefinitionMap[interfaceName];
      if (typeDefinition) interfaceTypes.push(typeDefinition);
      var interfaceDefinesField = interfaceTypes.some(function (type) {
        return type.fields.some(function (field) {
          return field.name.value === fieldName;
        });
      });

      if (interfaceDefinesField) {
        isImplementedField = true;
        definingType = interfaceName;
      }
    });
  }

  return [definingType, isImplementedField];
};
/**
 * Given a relationship type field, builds the input value
 * definitions for its Query arguments, along with those needed
 * for input types generated to support the same Query API
 * for the given field of the given relationship type
 */


var augmentRelationshipTypeFieldInput = function augmentRelationshipTypeFieldInput(_ref3) {
  var typeName = _ref3.typeName,
      relatedType = _ref3.relatedType,
      fieldArguments = _ref3.fieldArguments,
      fieldName = _ref3.fieldName,
      isImplementedField = _ref3.isImplementedField,
      outputType = _ref3.outputType,
      fromType = _ref3.fromType,
      toType = _ref3.toType,
      typeDefinitionMap = _ref3.typeDefinitionMap,
      typeExtensionDefinitionMap = _ref3.typeExtensionDefinitionMap,
      generatedTypeMap = _ref3.generatedTypeMap,
      nodeInputTypeMap = _ref3.nodeInputTypeMap,
      relationshipInputTypeMap = _ref3.relationshipInputTypeMap,
      outputTypeWrappers = _ref3.outputTypeWrappers,
      config = _ref3.config;

  if (!(0, _federation.isExternalTypeExtension)({
    typeName: fromType,
    typeMap: typeDefinitionMap,
    typeExtensionDefinitionMap: typeExtensionDefinitionMap
  }) && !(0, _federation.isExternalTypeExtension)({
    typeName: toType,
    typeMap: typeDefinitionMap,
    typeExtensionDefinitionMap: typeExtensionDefinitionMap
  })) {
    var _nodeInputTypeMap$Fil;

    var relationshipFilterTypeName = "_".concat(typeName).concat(outputType[0].toUpperCase() + outputType.substr(1)); // Assume outgoing relationship

    if (fromType === toType) {
      relationshipFilterTypeName = "_".concat(outputType, "Directions");
    }

    (_nodeInputTypeMap$Fil = nodeInputTypeMap[_inputValues.FilteringArgument.FILTER].fields).push.apply(_nodeInputTypeMap$Fil, (0, _toConsumableArray2["default"])(buildRelationshipFilters({
      typeName: typeName,
      fieldName: fieldName,
      outputType: "".concat(relationshipFilterTypeName, "Filter"),
      relatedType: outputType,
      outputTypeWrappers: outputTypeWrappers,
      config: config
    })));

    var _augmentRelationshipT3 = augmentRelationshipTypeFieldArguments({
      fieldArguments: fieldArguments,
      typeName: typeName,
      fromType: fromType,
      toType: toType,
      isImplementedField: isImplementedField,
      outputType: outputType,
      relatedType: relatedType,
      relationshipFilterTypeName: relationshipFilterTypeName,
      outputTypeWrappers: outputTypeWrappers,
      typeDefinitionMap: typeDefinitionMap,
      generatedTypeMap: generatedTypeMap,
      relationshipInputTypeMap: relationshipInputTypeMap
    });

    var _augmentRelationshipT4 = (0, _slicedToArray2["default"])(_augmentRelationshipT3, 2);

    fieldArguments = _augmentRelationshipT4[0];
    generatedTypeMap = _augmentRelationshipT4[1];
  }

  return [fieldArguments, generatedTypeMap, nodeInputTypeMap];
};
/**
 * Builds the AST for the input value definitions used for
 * relationship type Query field arguments
 */


var augmentRelationshipTypeFieldArguments = function augmentRelationshipTypeFieldArguments(_ref4) {
  var fieldArguments = _ref4.fieldArguments,
      typeName = _ref4.typeName,
      fromType = _ref4.fromType,
      toType = _ref4.toType,
      isImplementedField = _ref4.isImplementedField,
      outputType = _ref4.outputType,
      relatedType = _ref4.relatedType,
      relationshipFilterTypeName = _ref4.relationshipFilterTypeName,
      outputTypeWrappers = _ref4.outputTypeWrappers,
      typeDefinitionMap = _ref4.typeDefinitionMap,
      generatedTypeMap = _ref4.generatedTypeMap,
      relationshipInputTypeMap = _ref4.relationshipInputTypeMap;

  if (fromType !== toType) {
    fieldArguments = (0, _inputValues.buildQueryFieldArguments)({
      argumentMap: RelationshipQueryArgument,
      fieldArguments: fieldArguments,
      typeName: typeName,
      outputType: outputType,
      outputTypeWrappers: outputTypeWrappers,
      typeDefinitionMap: typeDefinitionMap
    });
  } else {
    fieldArguments = [];
  }

  if (!isImplementedField) {
    // If this relationship type field is on an object type implementing an
    // interface that defines it, then the argument input types and output type
    // must be those used on that interface's field definition
    generatedTypeMap = buildRelationshipSelectionArgumentInputTypes({
      fromType: fromType,
      toType: toType,
      relatedType: relatedType,
      relationshipFilterTypeName: relationshipFilterTypeName,
      generatedTypeMap: generatedTypeMap,
      relationshipInputTypeMap: relationshipInputTypeMap,
      typeDefinitionMap: typeDefinitionMap
    });
  }

  return [fieldArguments, generatedTypeMap];
};
/**
 * Builds the AST for object type definitions used for transforming
 * a relationship type field on a node type - will likely not be
 * necessary once we allow for dynamically named fields for the
 * 'from' and 'to' node type reference fields on relationship types
 */


var transformRelationshipTypeFieldOutput = function transformRelationshipTypeFieldOutput(_ref5) {
  var typeName = _ref5.typeName,
      relatedType = _ref5.relatedType,
      fieldArguments = _ref5.fieldArguments,
      fieldName = _ref5.fieldName,
      outputType = _ref5.outputType,
      fromType = _ref5.fromType,
      toType = _ref5.toType,
      typeDefinitionMap = _ref5.typeDefinitionMap,
      generatedTypeMap = _ref5.generatedTypeMap,
      outputTypeWrappers = _ref5.outputTypeWrappers,
      relationshipName = _ref5.relationshipName,
      fieldType = _ref5.fieldType,
      propertyOutputFields = _ref5.propertyOutputFields;
  var relationshipOutputName = "_".concat(typeName).concat(fieldName[0].toUpperCase() + fieldName.substr(1));
  var unwrappedType = (0, _fields.unwrapNamedType)({
    type: fieldType
  });

  if (fromType === toType) {
    // Clear arguments on this field, given their distribution
    fieldType = (0, _ast.buildNamedType)({
      name: "".concat(relationshipOutputName, "Directions")
    });
  } else {
    // Output transform
    unwrappedType.name = relationshipOutputName;
    fieldType = (0, _ast.buildNamedType)(unwrappedType);
  }

  generatedTypeMap = buildRelationshipFieldOutputTypes({
    outputType: outputType,
    fromType: fromType,
    toType: toType,
    outputTypeWrappers: outputTypeWrappers,
    fieldArguments: fieldArguments,
    relationshipOutputName: relationshipOutputName,
    relationshipName: relationshipName,
    relatedType: relatedType,
    propertyOutputFields: propertyOutputFields,
    typeDefinitionMap: typeDefinitionMap,
    generatedTypeMap: generatedTypeMap
  });
  return [fieldType, generatedTypeMap];
};
/**
 * Builds the AST definitions that compose the Query filtering input type
 * values for a given relationship field
 */


var buildRelationshipFilters = function buildRelationshipFilters(_ref6) {
  var typeName = _ref6.typeName,
      fieldName = _ref6.fieldName,
      outputType = _ref6.outputType,
      relatedType = _ref6.relatedType,
      outputTypeWrappers = _ref6.outputTypeWrappers,
      config = _ref6.config;
  var filters = [];

  var queryTypeNameLower = _types.OperationType.QUERY.toLowerCase();

  if ((0, _augment.shouldAugmentRelationshipField)(config, queryTypeNameLower, typeName, relatedType)) {
    if ((0, _fields.isListTypeField)({
      wrappers: outputTypeWrappers
    })) {
      filters = (0, _inputValues.buildFilters)({
        fieldName: fieldName,
        fieldConfig: {
          name: fieldName,
          type: {
            name: outputType
          }
        },
        filterTypes: ['not', 'in', 'not_in', 'some', 'none', 'single', 'every']
      });
    } else {
      filters = (0, _inputValues.buildFilters)({
        fieldName: fieldName,
        fieldConfig: {
          name: fieldName,
          type: {
            name: outputType
          }
        },
        filterTypes: ['not', 'in', 'not_in']
      });
    }
  }

  return filters;
};
/**
 * Builds the AST definitions for the incoming and outgoing node type
 * fields of the output object types generated for querying relationship
 * type fields
 */


exports.buildRelationshipFilters = buildRelationshipFilters;

var buildNodeOutputFields = function buildNodeOutputFields(_ref7) {
  var fromType = _ref7.fromType,
      toType = _ref7.toType,
      _ref7$args = _ref7.args,
      args = _ref7$args === void 0 ? [] : _ref7$args,
      _ref7$wrappers = _ref7.wrappers,
      wrappers = _ref7$wrappers === void 0 ? {} : _ref7$wrappers;
  return [(0, _ast.buildField)({
    name: (0, _ast.buildName)({
      name: _relationship.RelationshipDirectionField.FROM
    }),
    args: args,
    type: (0, _ast.buildNamedType)({
      name: fromType,
      wrappers: wrappers
    })
  }), (0, _ast.buildField)({
    name: (0, _ast.buildName)({
      name: _relationship.RelationshipDirectionField.TO
    }),
    args: args,
    type: (0, _ast.buildNamedType)({
      name: toType,
      wrappers: wrappers
    })
  })];
};
/**
 * Builds the AST definitions for the object types generated
 * for querying relationship type fields on node types
 */


exports.buildNodeOutputFields = buildNodeOutputFields;

var buildRelationshipFieldOutputTypes = function buildRelationshipFieldOutputTypes(_ref8) {
  var outputType = _ref8.outputType,
      fromType = _ref8.fromType,
      toType = _ref8.toType,
      outputTypeWrappers = _ref8.outputTypeWrappers,
      fieldArguments = _ref8.fieldArguments,
      relationshipOutputName = _ref8.relationshipOutputName,
      relationshipName = _ref8.relationshipName,
      relatedType = _ref8.relatedType,
      propertyOutputFields = _ref8.propertyOutputFields,
      typeDefinitionMap = _ref8.typeDefinitionMap,
      generatedTypeMap = _ref8.generatedTypeMap;
  var relationTypeDirective = (0, _directives.buildRelationDirective)({
    relationshipName: relationshipName,
    fromType: fromType,
    toType: toType
  });

  if (fromType === toType) {
    fieldArguments = (0, _inputValues.buildQueryFieldArguments)({
      argumentMap: RelationshipQueryArgument,
      fieldArguments: fieldArguments,
      outputType: outputType,
      outputTypeWrappers: outputTypeWrappers,
      typeDefinitionMap: typeDefinitionMap
    });
    var reflexiveOutputName = "".concat(relationshipOutputName, "Directions");
    generatedTypeMap[reflexiveOutputName] = (0, _ast.buildObjectType)({
      name: (0, _ast.buildName)({
        name: reflexiveOutputName
      }),
      fields: buildNodeOutputFields({
        fromType: relationshipOutputName,
        toType: relationshipOutputName,
        args: fieldArguments,
        wrappers: (0, _defineProperty3["default"])({}, _fields.TypeWrappers.LIST_TYPE, true)
      }),
      directives: [relationTypeDirective]
    });
  }

  generatedTypeMap[relationshipOutputName] = (0, _ast.buildObjectType)({
    name: (0, _ast.buildName)({
      name: relationshipOutputName
    }),
    fields: [].concat((0, _toConsumableArray2["default"])(propertyOutputFields), [(0, _ast.buildField)({
      name: (0, _ast.buildName)({
        name: relatedType
      }),
      type: (0, _ast.buildNamedType)({
        name: relatedType
      })
    })]),
    directives: [relationTypeDirective]
  });
  return generatedTypeMap;
};
/**
 * Given information about a field on a relationship type, builds
 * the AST for associated input value definitions used by input
 * types generated for the Query API
 */


var buildRelationshipSelectionArgumentInputTypes = function buildRelationshipSelectionArgumentInputTypes(_ref9) {
  var fromType = _ref9.fromType,
      toType = _ref9.toType,
      relatedType = _ref9.relatedType,
      relationshipFilterTypeName = _ref9.relationshipFilterTypeName,
      generatedTypeMap = _ref9.generatedTypeMap,
      relationshipInputTypeMap = _ref9.relationshipInputTypeMap,
      typeDefinitionMap = _ref9.typeDefinitionMap;
  var relationshipFilteringFields = relationshipInputTypeMap[_inputValues.FilteringArgument.FILTER].fields;
  var relatedTypeFilterName = relationshipInputTypeMap[_inputValues.FilteringArgument.FILTER].name;

  if (fromType === toType) {
    var reflexiveFilteringTypeName = "".concat(relationshipFilterTypeName, "Filter");
    generatedTypeMap[reflexiveFilteringTypeName] = (0, _ast.buildInputObjectType)({
      name: (0, _ast.buildName)({
        name: reflexiveFilteringTypeName
      }),
      fields: buildNodeInputFields({
        fromType: relatedTypeFilterName,
        toType: relatedTypeFilterName
      })
    });
  }

  var relatedTypeFilteringField = (0, _ast.buildInputValue)({
    name: (0, _ast.buildName)({
      name: relatedType
    }),
    type: (0, _ast.buildNamedType)({
      name: "_".concat(relatedType, "Filter")
    })
  });
  relationshipFilteringFields.push(relatedTypeFilteringField);
  generatedTypeMap = (0, _inputValues.buildQueryFilteringInputType)({
    typeName: relatedTypeFilterName,
    typeDefinitionMap: typeDefinitionMap,
    generatedTypeMap: generatedTypeMap,
    inputTypeMap: relationshipInputTypeMap
  });
  generatedTypeMap = (0, _inputValues.buildQueryOrderingEnumType)({
    nodeInputTypeMap: relationshipInputTypeMap,
    typeDefinitionMap: typeDefinitionMap,
    generatedTypeMap: generatedTypeMap
  });
  return generatedTypeMap;
};
/**
 * Builds the AST definitions for the input values of the
 * incoming and outgoing nodes, used as relationship mutation
 * field arguments for selecting the related nodes
 */


var buildNodeInputFields = function buildNodeInputFields(_ref10) {
  var fromType = _ref10.fromType,
      toType = _ref10.toType;
  return [(0, _ast.buildInputValue)({
    name: (0, _ast.buildName)({
      name: _relationship.RelationshipDirectionField.FROM
    }),
    type: (0, _ast.buildNamedType)({
      name: fromType
    })
  }), (0, _ast.buildInputValue)({
    name: (0, _ast.buildName)({
      name: _relationship.RelationshipDirectionField.TO
    }),
    type: (0, _ast.buildNamedType)({
      name: toType
    })
  })];
};
/**
 * Given the name of a type, and the names of the node types
 * of a relationship type, decides which type it is related to
 * (possibly itself)
 */


var decideRelatedType = function decideRelatedType(_ref11) {
  var typeName = _ref11.typeName,
      fromType = _ref11.fromType,
      toType = _ref11.toType;
  var relatedType = toType;

  if (fromType !== toType) {
    // Interpret relationship direction
    if (typeName === toType) {
      // Is incoming relationship
      relatedType = fromType;
    }
  }

  return relatedType;
};