"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs2/helpers/interopRequireDefault");

var _Object$defineProperty2 = require("@babel/runtime-corejs2/core-js/object/define-property");

_Object$defineProperty2(exports, "__esModule", {
  value: true
});

exports.translateMutation = exports.translateQuery = exports.neo4jType = exports.neo4jTypeField = exports.nodeTypeFieldOnRelationType = exports.relationTypeFieldOnNodeType = exports.relationFieldOnNodeType = exports.customCypherField = exports.derivedTypesParams = exports.fragmentType = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/define-property"));

var _defineProperties = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/define-properties"));

var _getOwnPropertyDescriptors = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/get-own-property-descriptors"));

var _getOwnPropertyDescriptor = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/get-own-property-descriptor"));

var _getOwnPropertySymbols = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/get-own-property-symbols"));

var _isInteger = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/number/is-integer"));

var _entries = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/entries"));

var _keys = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/keys"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/toConsumableArray"));

var _isArray = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/array/is-array"));

var _defineProperty3 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/defineProperty"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/slicedToArray"));

var _utils = require("./utils");

var _graphql = require("graphql");

var _selections = require("./selections");

var _lodash = _interopRequireDefault(require("lodash"));

var _neo4jDriver = _interopRequireDefault(require("neo4j-driver"));

var _types = require("./augment/types/types");

var _federation = require("./federation");

var _fields = require("./augment/fields");

function ownKeys(object, enumerableOnly) { var keys = (0, _keys["default"])(object); if (_getOwnPropertySymbols["default"]) { var symbols = (0, _getOwnPropertySymbols["default"])(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return (0, _getOwnPropertyDescriptor["default"])(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty3["default"])(target, key, source[key]); }); } else if (_getOwnPropertyDescriptors["default"]) { (0, _defineProperties["default"])(target, (0, _getOwnPropertyDescriptors["default"])(source)); } else { ownKeys(Object(source)).forEach(function (key) { (0, _defineProperty2["default"])(target, key, (0, _getOwnPropertyDescriptor["default"])(source, key)); }); } } return target; }

var derivedTypesParamName = function derivedTypesParamName(schemaTypeName) {
  return "".concat(schemaTypeName, "_derivedTypes");
};

var fragmentType = function fragmentType(varName, schemaTypeName) {
  return "FRAGMENT_TYPE: head( [ label IN labels(".concat(varName, ") WHERE label IN $").concat(derivedTypesParamName(schemaTypeName), " ] )");
};

exports.fragmentType = fragmentType;

var derivedTypesParams = function derivedTypesParams(_ref) {
  var isInterfaceType = _ref.isInterfaceType,
      isUnionType = _ref.isUnionType,
      schema = _ref.schema,
      schemaTypeName = _ref.schemaTypeName,
      usesFragments = _ref.usesFragments;
  var params = {};

  if (!usesFragments) {
    if (isInterfaceType) {
      var paramName = derivedTypesParamName(schemaTypeName);
      params[paramName] = (0, _utils.getInterfaceDerivedTypeNames)(schema, schemaTypeName);
    } else if (isUnionType) {
      var _paramName = derivedTypesParamName(schemaTypeName);

      var typeMap = schema.getTypeMap();
      var schemaType = typeMap[schemaTypeName];
      var types = schemaType.getTypes();
      params[_paramName] = types.map(function (type) {
        return type.name;
      });
    }
  }

  return params;
};

exports.derivedTypesParams = derivedTypesParams;

var customCypherField = function customCypherField(_ref2) {
  var customCypherStatement = _ref2.customCypherStatement,
      cypherParams = _ref2.cypherParams,
      paramIndex = _ref2.paramIndex,
      schemaTypeRelation = _ref2.schemaTypeRelation,
      isObjectTypeField = _ref2.isObjectTypeField,
      isInterfaceTypeField = _ref2.isInterfaceTypeField,
      isUnionTypeField = _ref2.isUnionTypeField,
      usesFragments = _ref2.usesFragments,
      schemaTypeFields = _ref2.schemaTypeFields,
      derivedTypeMap = _ref2.derivedTypeMap,
      initial = _ref2.initial,
      fieldName = _ref2.fieldName,
      fieldType = _ref2.fieldType,
      nestedVariable = _ref2.nestedVariable,
      variableName = _ref2.variableName,
      headSelection = _ref2.headSelection,
      schemaType = _ref2.schemaType,
      innerSchemaType = _ref2.innerSchemaType,
      resolveInfo = _ref2.resolveInfo,
      subSelection = _ref2.subSelection,
      skipLimit = _ref2.skipLimit,
      commaIfTail = _ref2.commaIfTail,
      tailParams = _ref2.tailParams,
      isFederatedOperation = _ref2.isFederatedOperation,
      context = _ref2.context;

  var _buildMapProjection = buildMapProjection({
    isComputedField: true,
    schemaType: innerSchemaType,
    isObjectType: isObjectTypeField,
    isInterfaceType: isInterfaceTypeField,
    isUnionType: isUnionTypeField,
    usesFragments: usesFragments,
    safeVariableName: nestedVariable,
    subQuery: subSelection[0],
    schemaTypeFields: schemaTypeFields,
    derivedTypeMap: derivedTypeMap,
    resolveInfo: resolveInfo
  }),
      _buildMapProjection2 = (0, _slicedToArray2["default"])(_buildMapProjection, 2),
      mapProjection = _buildMapProjection2[0],
      labelPredicate = _buildMapProjection2[1];

  var headListWrapperPrefix = "".concat(!(0, _utils.isArrayType)(fieldType) ? 'head(' : '');
  var headListWrapperSuffix = "".concat(!(0, _utils.isArrayType)(fieldType) ? ')' : ''); // For @cypher fields with object payload types, customCypherField is
  // called after the recursive call to compute a subSelection. But recurse()
  // increments paramIndex. So here we need to decrement it in order to map
  // appropriately to the indexed keys produced in getFilterParams()

  var cypherFieldParamsIndex = paramIndex - 1;

  if (schemaTypeRelation) {
    variableName = "".concat(variableName, "_relation");
  }

  return _objectSpread({
    initial: "".concat(initial).concat(fieldName, ": ").concat(headListWrapperPrefix).concat(labelPredicate ? "[".concat(nestedVariable, " IN ") : '', "[ ").concat(nestedVariable, " IN apoc.cypher.runFirstColumn(\"").concat(customCypherStatement, "\", {").concat((0, _utils.cypherDirectiveArgs)(variableName, headSelection, cypherParams, schemaType, resolveInfo, cypherFieldParamsIndex, isFederatedOperation, context), "}, true) ").concat(labelPredicate, "| ").concat(labelPredicate ? "".concat(nestedVariable, "] | ") : '').concat(mapProjection, "]").concat(headListWrapperSuffix).concat(skipLimit, " ").concat(commaIfTail)
  }, tailParams);
};

exports.customCypherField = customCypherField;

var relationFieldOnNodeType = function relationFieldOnNodeType(_ref3) {
  var initial = _ref3.initial,
      fieldName = _ref3.fieldName,
      fieldType = _ref3.fieldType,
      fieldSelectionSet = _ref3.fieldSelectionSet,
      variableName = _ref3.variableName,
      relDirection = _ref3.relDirection,
      relType = _ref3.relType,
      nestedVariable = _ref3.nestedVariable,
      schemaTypeFields = _ref3.schemaTypeFields,
      derivedTypeMap = _ref3.derivedTypeMap,
      isObjectTypeField = _ref3.isObjectTypeField,
      isInterfaceTypeField = _ref3.isInterfaceTypeField,
      isUnionTypeField = _ref3.isUnionTypeField,
      usesFragments = _ref3.usesFragments,
      innerSchemaType = _ref3.innerSchemaType,
      paramIndex = _ref3.paramIndex,
      fieldArgs = _ref3.fieldArgs,
      filterParams = _ref3.filterParams,
      selectionFilters = _ref3.selectionFilters,
      neo4jTypeArgs = _ref3.neo4jTypeArgs,
      fieldsForTranslation = _ref3.fieldsForTranslation,
      schemaType = _ref3.schemaType,
      subSelection = _ref3.subSelection,
      skipLimit = _ref3.skipLimit,
      commaIfTail = _ref3.commaIfTail,
      tailParams = _ref3.tailParams,
      resolveInfo = _ref3.resolveInfo,
      cypherParams = _ref3.cypherParams;
  var safeVariableName = (0, _utils.safeVar)(nestedVariable);
  var allParams = (0, _utils.innerFilterParams)(filterParams, neo4jTypeArgs);
  var queryParams = (0, _utils.paramsToString)(_lodash["default"].filter(allParams, function (param) {
    return !(0, _isArray["default"])(param.value);
  }));

  var _processFilterArgumen = processFilterArgument({
    fieldArgs: fieldArgs,
    schemaType: innerSchemaType,
    variableName: nestedVariable,
    resolveInfo: resolveInfo,
    params: selectionFilters,
    paramIndex: paramIndex
  }),
      _processFilterArgumen2 = (0, _slicedToArray2["default"])(_processFilterArgumen, 2),
      filterPredicates = _processFilterArgumen2[0],
      serializedFilterParam = _processFilterArgumen2[1];

  var filterParamKey = "".concat(tailParams.paramIndex, "_filter");
  var fieldArgumentParams = subSelection[1];
  var filterParam = fieldArgumentParams[filterParamKey];

  if (filterParam && typeof serializedFilterParam[filterParamKey] !== 'undefined') {
    subSelection[1][filterParamKey] = serializedFilterParam[filterParamKey];
  }

  var arrayFilterParams = _lodash["default"].pickBy(filterParams, function (param, keyName) {
    return (0, _isArray["default"])(param.value) && !('orderBy' === keyName);
  });

  var neo4jTypeClauses = (0, _utils.neo4jTypePredicateClauses)(filterParams, nestedVariable, neo4jTypeArgs);

  var arrayPredicates = _lodash["default"].map(arrayFilterParams, function (value, key) {
    var param = _lodash["default"].find(allParams, function (param) {
      return param.key === key;
    });

    return "".concat(safeVariableName, ".").concat((0, _utils.safeVar)(key), " IN $").concat(param.value.index, "_").concat(key);
  });

  var subQuery = subSelection[0];

  var _buildMapProjection3 = buildMapProjection({
    schemaType: innerSchemaType,
    isObjectType: isObjectTypeField,
    isInterfaceType: isInterfaceTypeField,
    isUnionType: isUnionTypeField,
    usesFragments: usesFragments,
    safeVariableName: safeVariableName,
    subQuery: subQuery,
    schemaTypeFields: schemaTypeFields,
    derivedTypeMap: derivedTypeMap,
    resolveInfo: resolveInfo
  }),
      _buildMapProjection4 = (0, _slicedToArray2["default"])(_buildMapProjection3, 2),
      mapProjection = _buildMapProjection4[0],
      labelPredicate = _buildMapProjection4[1];

  subSelection[1] = _objectSpread({}, subSelection[1]);
  var whereClauses = [labelPredicate].concat((0, _toConsumableArray2["default"])(neo4jTypeClauses), (0, _toConsumableArray2["default"])(arrayPredicates), (0, _toConsumableArray2["default"])(filterPredicates)).filter(function (predicate) {
    return !!predicate;
  });
  var orderByParam = filterParams['orderBy'];
  var usesTemporalOrdering = temporalOrderingFieldExists(schemaType, filterParams);
  var selectedFieldNames = fieldSelectionSet.reduce(function (fieldNames, field) {
    if (field.name) fieldNames.push(field.name.value);
    return fieldNames;
  }, []);
  var neo4jTypeFieldSelections = buildOrderedNeo4jTypeSelections({
    schemaType: innerSchemaType,
    selections: fieldsForTranslation,
    usesTemporalOrdering: usesTemporalOrdering,
    selectedFieldNames: selectedFieldNames
  });
  tailParams.initial = "".concat(initial).concat(fieldName, ": ").concat(!(0, _utils.isArrayType)(fieldType) ? 'head(' : '').concat(orderByParam ? usesTemporalOrdering ? "[sortedElement IN apoc.coll.sortMulti(" : "apoc.coll.sortMulti(" : '', "[(").concat((0, _utils.safeVar)(variableName), ")").concat(isUnionTypeField ? "--" : "".concat(relDirection === 'in' || relDirection === 'IN' ? '<' : '', "-[:").concat((0, _utils.safeLabel)([relType]), "]-").concat(relDirection === 'out' || relDirection === 'OUT' ? '>' : ''), "(").concat(safeVariableName, ":".concat((0, _utils.safeLabel)([innerSchemaType.name].concat((0, _toConsumableArray2["default"])((0, _utils.getAdditionalLabels)(resolveInfo.schema.getType(innerSchemaType.name), cypherParams)))))).concat(queryParams, ")").concat(whereClauses.length > 0 ? " WHERE ".concat(whereClauses.join(' AND ')) : '', " | ").concat(mapProjection, "]").concat(orderByParam ? ", [".concat(buildSortMultiArgs(orderByParam), "])").concat(usesTemporalOrdering ? " | sortedElement { .* ".concat(neo4jTypeFieldSelections ? ",  ".concat(neo4jTypeFieldSelections) : '', "}]") : "") : '').concat(!(0, _utils.isArrayType)(fieldType) ? ')' : '').concat(skipLimit, " ").concat(commaIfTail);
  return [tailParams, subSelection];
};

exports.relationFieldOnNodeType = relationFieldOnNodeType;

var relationTypeFieldOnNodeType = function relationTypeFieldOnNodeType(_ref4) {
  var innerSchemaTypeRelation = _ref4.innerSchemaTypeRelation,
      initial = _ref4.initial,
      fieldName = _ref4.fieldName,
      fieldSelectionSet = _ref4.fieldSelectionSet,
      subSelection = _ref4.subSelection,
      skipLimit = _ref4.skipLimit,
      commaIfTail = _ref4.commaIfTail,
      tailParams = _ref4.tailParams,
      fieldType = _ref4.fieldType,
      variableName = _ref4.variableName,
      fieldsForTranslation = _ref4.fieldsForTranslation,
      schemaType = _ref4.schemaType,
      innerSchemaType = _ref4.innerSchemaType,
      nestedVariable = _ref4.nestedVariable,
      queryParams = _ref4.queryParams,
      filterParams = _ref4.filterParams,
      neo4jTypeArgs = _ref4.neo4jTypeArgs,
      resolveInfo = _ref4.resolveInfo,
      selectionFilters = _ref4.selectionFilters,
      paramIndex = _ref4.paramIndex,
      fieldArgs = _ref4.fieldArgs,
      cypherParams = _ref4.cypherParams;
  var translation = '';

  if (innerSchemaTypeRelation.from === innerSchemaTypeRelation.to) {
    translation = "".concat(initial).concat(fieldName, ": {").concat(subSelection[0], "}").concat(skipLimit, " ").concat(commaIfTail);
  } else {
    var relationshipVariableName = "".concat(nestedVariable, "_relation");
    var neo4jTypeClauses = (0, _utils.neo4jTypePredicateClauses)(filterParams, relationshipVariableName, neo4jTypeArgs);

    var _processFilterArgumen3 = processFilterArgument({
      fieldArgs: fieldArgs,
      parentSchemaType: schemaType,
      schemaType: innerSchemaType,
      variableName: relationshipVariableName,
      resolveInfo: resolveInfo,
      params: selectionFilters,
      paramIndex: paramIndex,
      rootIsRelationType: true
    }),
        _processFilterArgumen4 = (0, _slicedToArray2["default"])(_processFilterArgumen3, 2),
        filterPredicates = _processFilterArgumen4[0],
        serializedFilterParam = _processFilterArgumen4[1];

    var filterParamKey = "".concat(tailParams.paramIndex, "_filter");
    var fieldArgumentParams = subSelection[1];
    var filterParam = fieldArgumentParams[filterParamKey];

    if (filterParam && typeof serializedFilterParam[filterParamKey] !== 'undefined') {
      subSelection[1][filterParamKey] = serializedFilterParam[filterParamKey];
    }

    var whereClauses = [].concat((0, _toConsumableArray2["default"])(neo4jTypeClauses), (0, _toConsumableArray2["default"])(filterPredicates));
    var orderByParam = filterParams['orderBy'];
    var usesTemporalOrdering = temporalOrderingFieldExists(innerSchemaType, filterParams);
    var selectedFieldNames = fieldSelectionSet.reduce(function (fieldNames, field) {
      if (field.name) fieldNames.push(field.name.value);
      return fieldNames;
    }, []);
    var neo4jTypeFieldSelections = buildOrderedNeo4jTypeSelections({
      schemaType: innerSchemaType,
      selections: fieldsForTranslation,
      usesTemporalOrdering: usesTemporalOrdering,
      selectedFieldNames: selectedFieldNames
    });
    var lhsOrdering = "".concat(orderByParam ? usesTemporalOrdering ? "[sortedElement IN apoc.coll.sortMulti(" : "apoc.coll.sortMulti(" : '');
    var rhsOrdering = "".concat(orderByParam ? ", [".concat(buildSortMultiArgs(orderByParam), "])").concat(usesTemporalOrdering ? " | sortedElement { .* ".concat(neo4jTypeFieldSelections ? ",  ".concat(neo4jTypeFieldSelections) : '', "}]") : "") : '');
    var incomingNodeTypeName = innerSchemaTypeRelation.from;
    var outgoingNodeTypeName = innerSchemaTypeRelation.to;
    var innerSchemaTypeFields = innerSchemaType.getFields();
    var selectsIncomingField = innerSchemaTypeFields[incomingNodeTypeName];
    var selectsOutgoingField = innerSchemaTypeFields[outgoingNodeTypeName];
    translation = "".concat(initial).concat(fieldName, ": ").concat(!(0, _utils.isArrayType)(fieldType) ? 'head(' : '').concat(lhsOrdering, "[(").concat((0, _utils.safeVar)(variableName), ")").concat( // if its fromField -- is this logically equivalent?
    selectsIncomingField ? '<' : '', "-[").concat((0, _utils.safeVar)(relationshipVariableName), ":").concat((0, _utils.safeLabel)(innerSchemaTypeRelation.name)).concat(queryParams, "]-").concat(selectsOutgoingField ? '>' : '', "(:").concat((0, _utils.safeLabel)(selectsOutgoingField ? [outgoingNodeTypeName].concat((0, _toConsumableArray2["default"])((0, _utils.getAdditionalLabels)(resolveInfo.schema.getType(outgoingNodeTypeName), cypherParams))) : [incomingNodeTypeName].concat((0, _toConsumableArray2["default"])((0, _utils.getAdditionalLabels)(resolveInfo.schema.getType(incomingNodeTypeName), cypherParams)))), ") ").concat(whereClauses.length > 0 ? "WHERE ".concat(whereClauses.join(' AND '), " ") : '', "| ").concat(relationshipVariableName, " {").concat(subSelection[0], "}]").concat(rhsOrdering).concat(!(0, _utils.isArrayType)(fieldType) ? ')' : '').concat(skipLimit, " ").concat(commaIfTail);
  }

  tailParams.initial = translation;
  return [tailParams, subSelection];
};

exports.relationTypeFieldOnNodeType = relationTypeFieldOnNodeType;

var nodeTypeFieldOnRelationType = function nodeTypeFieldOnRelationType(_ref5) {
  var initial = _ref5.initial,
      fieldName = _ref5.fieldName,
      fieldType = _ref5.fieldType,
      variableName = _ref5.variableName,
      nestedVariable = _ref5.nestedVariable,
      queryParams = _ref5.queryParams,
      subSelection = _ref5.subSelection,
      skipLimit = _ref5.skipLimit,
      commaIfTail = _ref5.commaIfTail,
      tailParams = _ref5.tailParams,
      filterParams = _ref5.filterParams,
      neo4jTypeArgs = _ref5.neo4jTypeArgs,
      schemaTypeRelation = _ref5.schemaTypeRelation,
      innerSchemaType = _ref5.innerSchemaType,
      fieldSelectionSet = _ref5.fieldSelectionSet,
      fieldsForTranslation = _ref5.fieldsForTranslation,
      schemaTypeFields = _ref5.schemaTypeFields,
      derivedTypeMap = _ref5.derivedTypeMap,
      isObjectTypeField = _ref5.isObjectTypeField,
      isInterfaceTypeField = _ref5.isInterfaceTypeField,
      isUnionTypeField = _ref5.isUnionTypeField,
      usesFragments = _ref5.usesFragments,
      paramIndex = _ref5.paramIndex,
      parentSelectionInfo = _ref5.parentSelectionInfo,
      resolveInfo = _ref5.resolveInfo,
      selectionFilters = _ref5.selectionFilters,
      fieldArgs = _ref5.fieldArgs,
      cypherParams = _ref5.cypherParams;

  if ((0, _utils.isRootSelection)({
    selectionInfo: parentSelectionInfo,
    rootType: 'relationship'
  }) && (0, _utils.isRelationTypeDirectedField)(fieldName)) {
    var nodeFieldVariableName = decideRootRelationshipTypeNodeVariable({
      parentSelectionInfo: parentSelectionInfo,
      fieldName: fieldName
    });

    var _buildMapProjection5 = buildMapProjection({
      schemaType: innerSchemaType,
      isObjectType: isObjectTypeField,
      isInterfaceType: isInterfaceTypeField,
      isUnionType: isUnionTypeField,
      safeVariableName: nodeFieldVariableName,
      subQuery: subSelection[0],
      usesFragments: usesFragments,
      schemaTypeFields: schemaTypeFields,
      derivedTypeMap: derivedTypeMap,
      resolveInfo: resolveInfo
    }),
        _buildMapProjection6 = (0, _slicedToArray2["default"])(_buildMapProjection5, 2),
        mapProjection = _buildMapProjection6[0],
        labelPredicate = _buildMapProjection6[1];

    var translationParams = relationTypeMutationPayloadField({
      initial: initial,
      fieldName: fieldName,
      mapProjection: mapProjection,
      skipLimit: skipLimit,
      commaIfTail: commaIfTail,
      tailParams: tailParams,
      parentSelectionInfo: parentSelectionInfo
    });
    return [translationParams, subSelection];
  } // Normal case of schemaType with a relationship directive


  return directedNodeTypeFieldOnRelationType({
    initial: initial,
    fieldName: fieldName,
    fieldType: fieldType,
    variableName: variableName,
    queryParams: queryParams,
    nestedVariable: nestedVariable,
    subSelection: subSelection,
    skipLimit: skipLimit,
    commaIfTail: commaIfTail,
    tailParams: tailParams,
    schemaTypeRelation: schemaTypeRelation,
    innerSchemaType: innerSchemaType,
    fieldSelectionSet: fieldSelectionSet,
    fieldsForTranslation: fieldsForTranslation,
    usesFragments: usesFragments,
    isObjectTypeField: isObjectTypeField,
    isInterfaceTypeField: isInterfaceTypeField,
    isUnionTypeField: isUnionTypeField,
    filterParams: filterParams,
    neo4jTypeArgs: neo4jTypeArgs,
    paramIndex: paramIndex,
    resolveInfo: resolveInfo,
    selectionFilters: selectionFilters,
    schemaTypeFields: schemaTypeFields,
    derivedTypeMap: derivedTypeMap,
    fieldArgs: fieldArgs,
    cypherParams: cypherParams
  });
};

exports.nodeTypeFieldOnRelationType = nodeTypeFieldOnRelationType;

var decideRootRelationshipTypeNodeVariable = function decideRootRelationshipTypeNodeVariable(_ref6) {
  var _ref6$parentSelection = _ref6.parentSelectionInfo,
      parentSelectionInfo = _ref6$parentSelection === void 0 ? {} : _ref6$parentSelection,
      fieldName = _ref6.fieldName;
  var fromVariable = parentSelectionInfo.from;
  var toVariable = parentSelectionInfo.to; // assume incoming

  var variableName = (0, _utils.safeVar)(fromVariable); // else set as outgoing

  if (fieldName === 'to') variableName = (0, _utils.safeVar)(toVariable);
  return variableName;
};

var relationTypeMutationPayloadField = function relationTypeMutationPayloadField(_ref7) {
  var initial = _ref7.initial,
      fieldName = _ref7.fieldName,
      mapProjection = _ref7.mapProjection,
      skipLimit = _ref7.skipLimit,
      commaIfTail = _ref7.commaIfTail,
      tailParams = _ref7.tailParams,
      parentSelectionInfo = _ref7.parentSelectionInfo;
  return _objectSpread({
    initial: "".concat(initial).concat(fieldName, ": ").concat(mapProjection).concat(skipLimit, " ").concat(commaIfTail)
  }, tailParams, {
    variableName: fieldName === 'from' ? parentSelectionInfo.to : parentSelectionInfo.from
  });
};

var directedNodeTypeFieldOnRelationType = function directedNodeTypeFieldOnRelationType(_ref8) {
  var initial = _ref8.initial,
      fieldName = _ref8.fieldName,
      fieldType = _ref8.fieldType,
      variableName = _ref8.variableName,
      queryParams = _ref8.queryParams,
      nestedVariable = _ref8.nestedVariable,
      subSelection = _ref8.subSelection,
      skipLimit = _ref8.skipLimit,
      commaIfTail = _ref8.commaIfTail,
      tailParams = _ref8.tailParams,
      schemaTypeRelation = _ref8.schemaTypeRelation,
      innerSchemaType = _ref8.innerSchemaType,
      fieldSelectionSet = _ref8.fieldSelectionSet,
      fieldsForTranslation = _ref8.fieldsForTranslation,
      usesFragments = _ref8.usesFragments,
      isObjectTypeField = _ref8.isObjectTypeField,
      isInterfaceTypeField = _ref8.isInterfaceTypeField,
      isUnionTypeField = _ref8.isUnionTypeField,
      filterParams = _ref8.filterParams,
      neo4jTypeArgs = _ref8.neo4jTypeArgs,
      paramIndex = _ref8.paramIndex,
      resolveInfo = _ref8.resolveInfo,
      selectionFilters = _ref8.selectionFilters,
      schemaTypeFields = _ref8.schemaTypeFields,
      derivedTypeMap = _ref8.derivedTypeMap,
      fieldArgs = _ref8.fieldArgs,
      cypherParams = _ref8.cypherParams;
  var relType = schemaTypeRelation.name;
  var fromTypeName = schemaTypeRelation.from;
  var toTypeName = schemaTypeRelation.to;
  var isFromField = fieldName === fromTypeName || fieldName === 'from';
  var isToField = fieldName === toTypeName || fieldName === 'to';
  var safeVariableName = nestedVariable;

  var _buildMapProjection7 = buildMapProjection({
    schemaType: innerSchemaType,
    isObjectType: isObjectTypeField,
    isInterfaceType: isInterfaceTypeField,
    isUnionType: isUnionTypeField,
    usesFragments: usesFragments,
    safeVariableName: safeVariableName,
    subQuery: subSelection[0],
    schemaTypeFields: schemaTypeFields,
    derivedTypeMap: derivedTypeMap,
    resolveInfo: resolveInfo
  }),
      _buildMapProjection8 = (0, _slicedToArray2["default"])(_buildMapProjection7, 2),
      mapProjection = _buildMapProjection8[0],
      labelPredicate = _buildMapProjection8[1]; // Since the translations are significantly different,
  // we first check whether the relationship is reflexive


  if (fromTypeName === toTypeName) {
    var relationshipVariableName = "".concat(variableName, "_").concat(isFromField ? 'from' : 'to', "_relation");

    if ((0, _utils.isRelationTypeDirectedField)(fieldName)) {
      var orderByParam = filterParams['orderBy'];
      var usesTemporalOrdering = temporalOrderingFieldExists(innerSchemaType, filterParams);
      var selectedFieldNames = fieldSelectionSet.reduce(function (fieldNames, field) {
        if (field.name) fieldNames.push(field.name.value);
        return fieldNames;
      }, []);
      var neo4jTypeFieldSelections = buildOrderedNeo4jTypeSelections({
        schemaType: innerSchemaType,
        selections: fieldsForTranslation,
        usesTemporalOrdering: usesTemporalOrdering,
        selectedFieldNames: selectedFieldNames
      });
      var lhsOrdering = "".concat(orderByParam ? usesTemporalOrdering ? "[sortedElement IN apoc.coll.sortMulti(" : "apoc.coll.sortMulti(" : '');
      var rhsOrdering = "".concat(orderByParam ? ", [".concat(buildSortMultiArgs(orderByParam), "])").concat(usesTemporalOrdering ? " | sortedElement { .* ".concat(neo4jTypeFieldSelections ? ",  ".concat(neo4jTypeFieldSelections) : '', "}]") : "") : '');
      var temporalFieldRelationshipVariableName = "".concat(nestedVariable, "_relation");
      var neo4jTypeClauses = (0, _utils.neo4jTypePredicateClauses)(filterParams, temporalFieldRelationshipVariableName, neo4jTypeArgs);

      var _processFilterArgumen5 = processFilterArgument({
        fieldArgs: fieldArgs,
        schemaType: innerSchemaType,
        variableName: relationshipVariableName,
        resolveInfo: resolveInfo,
        params: selectionFilters,
        paramIndex: paramIndex,
        rootIsRelationType: true
      }),
          _processFilterArgumen6 = (0, _slicedToArray2["default"])(_processFilterArgumen5, 2),
          filterPredicates = _processFilterArgumen6[0],
          serializedFilterParam = _processFilterArgumen6[1];

      var filterParamKey = "".concat(tailParams.paramIndex, "_filter");
      var fieldArgumentParams = subSelection[1];
      var filterParam = fieldArgumentParams[filterParamKey];

      if (filterParam && typeof serializedFilterParam[filterParamKey] !== 'undefined') {
        subSelection[1][filterParamKey] = serializedFilterParam[filterParamKey];
      }

      var whereClauses = [].concat((0, _toConsumableArray2["default"])(neo4jTypeClauses), (0, _toConsumableArray2["default"])(filterPredicates));
      tailParams.initial = "".concat(initial).concat(fieldName, ": ").concat(!(0, _utils.isArrayType)(fieldType) ? 'head(' : '').concat(lhsOrdering, "[(").concat((0, _utils.safeVar)(variableName), ")").concat(isFromField ? '<' : '', "-[").concat((0, _utils.safeVar)(relationshipVariableName), ":").concat((0, _utils.safeLabel)(relType)).concat(queryParams, "]-").concat(isToField ? '>' : '', "(").concat((0, _utils.safeVar)(nestedVariable), ":").concat((0, _utils.safeLabel)([fromTypeName].concat((0, _toConsumableArray2["default"])((0, _utils.getAdditionalLabels)(resolveInfo.schema.getType(fromTypeName), cypherParams)))), ") ").concat(whereClauses.length > 0 ? "WHERE ".concat(whereClauses.join(' AND '), " ") : '', "| ").concat(relationshipVariableName, " {").concat(subSelection[0], "}]").concat(rhsOrdering).concat(!(0, _utils.isArrayType)(fieldType) ? ')' : '').concat(skipLimit, " ").concat(commaIfTail);
      return [tailParams, subSelection];
    } else {
      // Case of a renamed directed field
      // e.g., 'from: Movie' -> 'Movie: Movie'
      tailParams.initial = "".concat(initial).concat(fieldName, ": ").concat(mapProjection).concat(skipLimit, " ").concat(commaIfTail);
      return [tailParams, subSelection];
    }
  } else {
    var _whereClauses = [labelPredicate].filter(function (predicate) {
      return !!predicate;
    });

    var safeRelationshipVar = (0, _utils.safeVar)("".concat(variableName, "_relation"));
    tailParams.initial = "".concat(initial).concat(fieldName, ": ").concat(!(0, _utils.isArrayType)(fieldType) ? 'head(' : '', "[(:").concat((0, _utils.safeLabel)(isFromField ? [toTypeName].concat((0, _toConsumableArray2["default"])((0, _utils.getAdditionalLabels)(resolveInfo.schema.getType(toTypeName), cypherParams))) : [fromTypeName].concat((0, _toConsumableArray2["default"])((0, _utils.getAdditionalLabels)(resolveInfo.schema.getType(fromTypeName), cypherParams)))), ")").concat(isUnionTypeField ? "--" : "".concat(isFromField ? '<' : '', "-[").concat(safeRelationshipVar, "]-").concat(isToField ? '>' : ''), "(").concat((0, _utils.safeVar)(nestedVariable), ":").concat((0, _utils.safeLabel)([innerSchemaType.name].concat((0, _toConsumableArray2["default"])((0, _utils.getAdditionalLabels)(resolveInfo.schema.getType(innerSchemaType.name), cypherParams))))).concat(queryParams, ")").concat(_whereClauses.length > 0 ? " WHERE ".concat(_whereClauses.join(' AND ')) : '', " | ").concat(mapProjection, "]").concat(!(0, _utils.isArrayType)(fieldType) ? ')' : '').concat(skipLimit, " ").concat(commaIfTail);
    return [tailParams, subSelection];
  }
};

var neo4jTypeField = function neo4jTypeField(_ref9) {
  var initial = _ref9.initial,
      fieldName = _ref9.fieldName,
      commaIfTail = _ref9.commaIfTail,
      tailParams = _ref9.tailParams,
      parentSelectionInfo = _ref9.parentSelectionInfo,
      secondParentSelectionInfo = _ref9.secondParentSelectionInfo;
  var parentFieldName = parentSelectionInfo.fieldName;
  var parentFieldType = parentSelectionInfo.fieldType;
  var parentSchemaType = parentSelectionInfo.schemaType;
  var parentVariableName = parentSelectionInfo.variableName;
  var secondParentVariableName = secondParentSelectionInfo.variableName; // Initially assume that the parent type of the temporal type
  // containing this temporal field was a node

  var variableName = parentVariableName;
  var fieldIsArray = (0, _utils.isArrayType)(parentFieldType);

  if (parentSchemaType && !(0, _utils.isNodeType)(parentSchemaType.astNode)) {
    // initial assumption wrong, build appropriate relationship variable
    if ((0, _utils.isRootSelection)({
      selectionInfo: secondParentSelectionInfo,
      rootType: 'relationship'
    })) {
      // If the second parent selection scope above is the root
      // then we need to use the root variableName
      variableName = "".concat(secondParentVariableName, "_relation");
    } else if ((0, _utils.isRelationTypePayload)(parentSchemaType)) {
      var parentSchemaTypeRelation = (0, _utils.getRelationTypeDirective)(parentSchemaType.astNode);

      if (parentSchemaTypeRelation.from === parentSchemaTypeRelation.to) {
        variableName = "".concat(variableName, "_relation");
      } else {
        variableName = "".concat(variableName, "_relation");
      }
    }
  }

  return _objectSpread({
    initial: "".concat(initial, " ").concat(fieldName, ": ").concat(fieldIsArray ? "".concat(fieldName === 'formatted' ? "toString(INSTANCE)" : "INSTANCE.".concat(fieldName), " ").concat(commaIfTail) : "".concat(fieldName === 'formatted' ? "toString(".concat((0, _utils.safeVar)(variableName), ".").concat(parentFieldName, ") ").concat(commaIfTail) : "".concat((0, _utils.safeVar)(variableName), ".").concat(parentFieldName, ".").concat(fieldName, " ").concat(commaIfTail)))
  }, tailParams);
};

exports.neo4jTypeField = neo4jTypeField;

var neo4jType = function neo4jType(_ref10) {
  var initial = _ref10.initial,
      fieldName = _ref10.fieldName,
      subSelection = _ref10.subSelection,
      commaIfTail = _ref10.commaIfTail,
      tailParams = _ref10.tailParams,
      variableName = _ref10.variableName,
      nestedVariable = _ref10.nestedVariable,
      fieldType = _ref10.fieldType,
      schemaType = _ref10.schemaType,
      schemaTypeRelation = _ref10.schemaTypeRelation,
      parentSelectionInfo = _ref10.parentSelectionInfo;
  var parentVariableName = parentSelectionInfo.variableName;
  var parentFilterParams = parentSelectionInfo.filterParams;
  var parentSchemaType = parentSelectionInfo.schemaType;
  var relationshipVariableSuffix = "relation";
  var fieldIsArray = (0, _utils.isArrayType)(fieldType);
  var isOrderedForNodeType = temporalOrderingFieldExists(parentSchemaType, parentFilterParams);
  var isOrderedForRelationshipType = temporalOrderingFieldExists(schemaType, parentFilterParams);

  if (!(0, _utils.isNodeType)(schemaType.astNode)) {
    if ((0, _utils.isRelationTypePayload)(schemaType) && schemaTypeRelation.from === schemaTypeRelation.to) {
      variableName = "".concat(nestedVariable, "_").concat(relationshipVariableSuffix);
    } else {
      if (fieldIsArray) {
        if ((0, _utils.isRootSelection)({
          selectionInfo: parentSelectionInfo,
          rootType: 'relationship'
        })) {
          variableName = "".concat(parentVariableName, "_").concat(relationshipVariableSuffix);
        } else {
          variableName = "".concat(variableName, "_").concat(relationshipVariableSuffix);
        }
      } else {
        if (isOrderedForRelationshipType) {
          variableName = "".concat(variableName, "_").concat(relationshipVariableSuffix);
        } else {
          variableName = "".concat(nestedVariable, "_").concat(relationshipVariableSuffix);
        }
      }
    }
  }

  var safeVariableName = (0, _utils.safeVar)(variableName);
  var usesTemporalOrdering = isOrderedForNodeType || isOrderedForRelationshipType;
  return _objectSpread({
    initial: "".concat(initial).concat(fieldName, ": ").concat(fieldIsArray ? "reduce(a = [], INSTANCE IN ".concat(variableName, ".").concat(fieldName, " | a + {").concat(subSelection[0], "})").concat(commaIfTail) : usesTemporalOrdering ? "".concat(safeVariableName, ".").concat(fieldName).concat(commaIfTail) : "{".concat(subSelection[0], "}").concat(commaIfTail))
  }, tailParams);
}; // Query API root operation branch


exports.neo4jType = neo4jType;

var translateQuery = function translateQuery(_ref11) {
  var resolveInfo = _ref11.resolveInfo,
      context = _ref11.context,
      first = _ref11.first,
      offset = _ref11.offset,
      _id = _ref11._id,
      orderBy = _ref11.orderBy,
      otherParams = _ref11.otherParams;

  var _typeIdentifiers = (0, _utils.typeIdentifiers)(resolveInfo.returnType),
      typeName = _typeIdentifiers.typeName,
      variableName = _typeIdentifiers.variableName;

  var schemaType = resolveInfo.schema.getType(typeName);
  var typeMap = resolveInfo.schema.getTypeMap();
  var selections = (0, _utils.getPayloadSelections)(resolveInfo);
  var isInterfaceType = (0, _utils.isGraphqlInterfaceType)(schemaType);
  var isUnionType = (0, _utils.isGraphqlUnionType)(schemaType);
  var isObjectType = (0, _utils.isGraphqlObjectType)(schemaType);

  var _filterNullParams = (0, _utils.filterNullParams)({
    offset: offset,
    first: first,
    otherParams: otherParams
  }),
      _filterNullParams2 = (0, _slicedToArray2["default"])(_filterNullParams, 2),
      nullParams = _filterNullParams2[0],
      nonNullParams = _filterNullParams2[1]; // Check is this is a federated operation, in which case get the lookup keys


  var operation = resolveInfo.operation || {}; // check if the operation name is the name used for generated queries

  var isFederatedOperation = operation.name && operation.name.value === _federation.NEO4j_GRAPHQL_SERVICE;
  var queryTypeCypherDirective = (0, _utils.getQueryCypherDirective)(resolveInfo, isFederatedOperation);
  var scalarKeys = {};
  var compoundKeys = {};
  var requiredData = {};

  if (isFederatedOperation) {
    var operationData = (0, _federation.getFederatedOperationData)({
      context: context
    });
    scalarKeys = operationData.scalarKeys;
    compoundKeys = operationData.compoundKeys;
    requiredData = operationData.requiredData;

    if (queryTypeCypherDirective) {
      // all nonnull keys become available as cypher variables
      nonNullParams = _objectSpread({}, scalarKeys, {}, compoundKeys, {}, requiredData);
    } else {
      // all scalar keys get used as field arguments, while relationship
      // field keys being translated as a filter argument
      nonNullParams = _objectSpread({}, scalarKeys);
    }
  }

  var filterParams = (0, _utils.getFilterParams)(nonNullParams);
  var queryArgs = (0, _utils.getQueryArguments)(resolveInfo, isFederatedOperation);
  var neo4jTypeArgs = (0, _utils.getNeo4jTypeArguments)(queryArgs);
  var cypherParams = getCypherParams(context);
  var queryParams = (0, _utils.paramsToString)((0, _utils.innerFilterParams)(filterParams, neo4jTypeArgs, null, queryTypeCypherDirective ? true : false), cypherParams);
  var safeVariableName = (0, _utils.safeVar)(variableName);
  var neo4jTypeClauses = (0, _utils.neo4jTypePredicateClauses)(filterParams, safeVariableName, neo4jTypeArgs);
  var outerSkipLimit = (0, _utils.getOuterSkipLimit)(first, offset);
  var orderByValue = (0, _utils.computeOrderBy)(resolveInfo, schemaType);
  var usesFragments = (0, _selections.isFragmentedSelection)({
    selections: selections
  });
  var isFragmentedInterfaceType = isInterfaceType && usesFragments;
  var isFragmentedObjectType = isObjectType && usesFragments;

  var _mergeSelectionFragme = (0, _selections.mergeSelectionFragments)({
    schemaType: schemaType,
    selections: selections,
    isFragmentedObjectType: isFragmentedObjectType,
    isUnionType: isUnionType,
    typeMap: typeMap,
    resolveInfo: resolveInfo
  }),
      _mergeSelectionFragme2 = (0, _slicedToArray2["default"])(_mergeSelectionFragme, 2),
      schemaTypeFields = _mergeSelectionFragme2[0],
      derivedTypeMap = _mergeSelectionFragme2[1];

  var hasOnlySchemaTypeFragments = schemaTypeFields.length > 0 && (0, _keys["default"])(derivedTypeMap).length === 0;
  if (hasOnlySchemaTypeFragments) usesFragments = false;

  if (queryTypeCypherDirective) {
    return customQuery({
      resolveInfo: resolveInfo,
      cypherParams: cypherParams,
      schemaType: schemaType,
      argString: queryParams,
      selections: selections,
      variableName: variableName,
      safeVariableName: safeVariableName,
      isObjectType: isObjectType,
      isInterfaceType: isInterfaceType,
      isUnionType: isUnionType,
      isFragmentedInterfaceType: isFragmentedInterfaceType,
      usesFragments: usesFragments,
      schemaTypeFields: schemaTypeFields,
      derivedTypeMap: derivedTypeMap,
      orderByValue: orderByValue,
      outerSkipLimit: outerSkipLimit,
      queryTypeCypherDirective: queryTypeCypherDirective,
      nonNullParams: nonNullParams
    });
  } else {
    var additionalLabels = (0, _utils.getAdditionalLabels)(schemaType, cypherParams);

    if (isFederatedOperation) {
      nonNullParams = (0, _federation.setCompoundKeyFilter)({
        params: nonNullParams,
        compoundKeys: compoundKeys
      });
      nonNullParams = _objectSpread({}, nonNullParams, {}, otherParams, {}, requiredData);
    }

    return nodeQuery({
      resolveInfo: resolveInfo,
      isFederatedOperation: isFederatedOperation,
      context: context,
      cypherParams: cypherParams,
      schemaType: schemaType,
      argString: queryParams,
      selections: selections,
      variableName: variableName,
      typeName: typeName,
      isObjectType: isObjectType,
      isInterfaceType: isInterfaceType,
      isUnionType: isUnionType,
      isFragmentedInterfaceType: isFragmentedInterfaceType,
      isFragmentedObjectType: isFragmentedObjectType,
      usesFragments: usesFragments,
      schemaTypeFields: schemaTypeFields,
      derivedTypeMap: derivedTypeMap,
      additionalLabels: additionalLabels,
      neo4jTypeClauses: neo4jTypeClauses,
      orderByValue: orderByValue,
      outerSkipLimit: outerSkipLimit,
      nullParams: nullParams,
      nonNullParams: nonNullParams,
      filterParams: filterParams,
      neo4jTypeArgs: neo4jTypeArgs,
      _id: _id
    });
  }
};

exports.translateQuery = translateQuery;

var buildTypeCompositionPredicate = function buildTypeCompositionPredicate(_ref12) {
  var schemaType = _ref12.schemaType,
      schemaTypeFields = _ref12.schemaTypeFields,
      _ref12$listVariable = _ref12.listVariable,
      listVariable = _ref12$listVariable === void 0 ? 'x' : _ref12$listVariable,
      derivedTypeMap = _ref12.derivedTypeMap,
      safeVariableName = _ref12.safeVariableName,
      isInterfaceType = _ref12.isInterfaceType,
      isUnionType = _ref12.isUnionType,
      isComputedQuery = _ref12.isComputedQuery,
      isComputedMutation = _ref12.isComputedMutation,
      isComputedField = _ref12.isComputedField,
      usesFragments = _ref12.usesFragments,
      resolveInfo = _ref12.resolveInfo;
  var schemaTypeName = schemaType.name;
  var isFragmentedInterfaceType = isInterfaceType && usesFragments;
  var labelPredicate = '';

  if (isFragmentedInterfaceType || isUnionType) {
    var derivedTypes = []; // If shared fields are selected then the translation builds
    // a type specific list comprehension for each interface implementing
    // type. Because of this, the type selecting predicate applied to
    // the interface type path pattern should allow for all possible
    // implementing types

    if (schemaTypeFields.length) {
      derivedTypes = (0, _selections.getDerivedTypes)({
        schemaTypeName: schemaTypeName,
        derivedTypeMap: derivedTypeMap,
        isUnionType: isUnionType,
        isFragmentedInterfaceType: isFragmentedInterfaceType,
        resolveInfo: resolveInfo
      });
    } else if (isUnionType) {
      derivedTypes = (0, _selections.getUnionDerivedTypes)({
        derivedTypeMap: derivedTypeMap,
        resolveInfo: resolveInfo
      });
    } else {
      // Otherwise, use only those types provided in fragments
      derivedTypes = (0, _keys["default"])(derivedTypeMap);
    }

    var typeSelectionPredicates = derivedTypes.map(function (selectedType) {
      return "\"".concat(selectedType, "\" IN labels(").concat(safeVariableName, ")");
    });

    if (typeSelectionPredicates.length) {
      labelPredicate = "(".concat(typeSelectionPredicates.join(' OR '), ")");
    }
  }

  if (labelPredicate) {
    if (isComputedQuery) {
      labelPredicate = "WITH [".concat(safeVariableName, " IN ").concat(listVariable, " WHERE ").concat(labelPredicate, " | ").concat(safeVariableName, "] AS ").concat(listVariable, " ");
    } else if (isComputedMutation) {
      labelPredicate = "UNWIND [".concat(safeVariableName, " IN ").concat(listVariable, " WHERE ").concat(labelPredicate, " | ").concat(safeVariableName, "] ");
    } else if (isComputedField) {
      labelPredicate = "WHERE ".concat(labelPredicate, " ");
    }
  }

  return labelPredicate;
};

var getCypherParams = function getCypherParams(context) {
  return context && context.cypherParams && context.cypherParams instanceof Object && (0, _keys["default"])(context.cypherParams).length > 0 ? context.cypherParams : undefined;
}; // Custom read operation


var customQuery = function customQuery(_ref13) {
  var resolveInfo = _ref13.resolveInfo,
      cypherParams = _ref13.cypherParams,
      schemaType = _ref13.schemaType,
      argString = _ref13.argString,
      selections = _ref13.selections,
      variableName = _ref13.variableName,
      isObjectType = _ref13.isObjectType,
      isInterfaceType = _ref13.isInterfaceType,
      isUnionType = _ref13.isUnionType,
      usesFragments = _ref13.usesFragments,
      schemaTypeFields = _ref13.schemaTypeFields,
      derivedTypeMap = _ref13.derivedTypeMap,
      orderByValue = _ref13.orderByValue,
      outerSkipLimit = _ref13.outerSkipLimit,
      queryTypeCypherDirective = _ref13.queryTypeCypherDirective,
      nonNullParams = _ref13.nonNullParams;
  var safeVariableName = (0, _utils.safeVar)(variableName);

  var _buildCypherSelection = (0, _selections.buildCypherSelection)({
    cypherParams: cypherParams,
    selections: selections,
    variableName: variableName,
    schemaType: schemaType,
    resolveInfo: resolveInfo
  }),
      _buildCypherSelection2 = (0, _slicedToArray2["default"])(_buildCypherSelection, 2),
      subQuery = _buildCypherSelection2[0],
      subParams = _buildCypherSelection2[1];

  var params = _objectSpread({}, nonNullParams, {}, subParams);

  if (cypherParams) {
    params['cypherParams'] = cypherParams;
  } // QueryType with a @cypher directive


  var cypherQueryArg = queryTypeCypherDirective.arguments.find(function (x) {
    return x.name.value === 'statement';
  });
  var isScalarType = (0, _utils.isGraphqlScalarType)(schemaType);
  var isNeo4jTypeOutput = (0, _utils.isNeo4jType)(schemaType.name);
  var orderByClause = orderByValue.cypherPart; // Don't add subQuery for scalar type payloads

  var isScalarPayload = isNeo4jTypeOutput || isScalarType;
  var fragmentTypeParams = derivedTypesParams({
    isInterfaceType: isInterfaceType,
    isUnionType: isUnionType,
    schema: resolveInfo.schema,
    schemaTypeName: schemaType.name,
    usesFragments: usesFragments
  });

  var _buildMapProjection9 = buildMapProjection({
    isComputedQuery: true,
    schemaType: schemaType,
    schemaTypeFields: schemaTypeFields,
    derivedTypeMap: derivedTypeMap,
    isObjectType: isObjectType,
    isInterfaceType: isInterfaceType,
    isUnionType: isUnionType,
    isScalarPayload: isScalarPayload,
    usesFragments: usesFragments,
    safeVariableName: safeVariableName,
    subQuery: subQuery,
    resolveInfo: resolveInfo
  }),
      _buildMapProjection10 = (0, _slicedToArray2["default"])(_buildMapProjection9, 2),
      mapProjection = _buildMapProjection10[0],
      labelPredicate = _buildMapProjection10[1];

  var query = "WITH apoc.cypher.runFirstColumn(\"".concat(cypherQueryArg.value.value, "\", ").concat(argString || 'null', ", True) AS x ").concat(labelPredicate, "UNWIND x AS ").concat(safeVariableName, " RETURN ").concat(isScalarPayload ? "".concat(mapProjection, " ") : "".concat(mapProjection, " AS ").concat(safeVariableName).concat(orderByClause)).concat(outerSkipLimit);
  return [query, _objectSpread({}, params, {}, fragmentTypeParams)];
}; // Generated API


var nodeQuery = function nodeQuery(_ref14) {
  var resolveInfo = _ref14.resolveInfo,
      isFederatedOperation = _ref14.isFederatedOperation,
      context = _ref14.context,
      cypherParams = _ref14.cypherParams,
      schemaType = _ref14.schemaType,
      selections = _ref14.selections,
      variableName = _ref14.variableName,
      typeName = _ref14.typeName,
      isObjectType = _ref14.isObjectType,
      isInterfaceType = _ref14.isInterfaceType,
      isUnionType = _ref14.isUnionType,
      usesFragments = _ref14.usesFragments,
      schemaTypeFields = _ref14.schemaTypeFields,
      derivedTypeMap = _ref14.derivedTypeMap,
      _ref14$additionalLabe = _ref14.additionalLabels,
      additionalLabels = _ref14$additionalLabe === void 0 ? [] : _ref14$additionalLabe,
      neo4jTypeClauses = _ref14.neo4jTypeClauses,
      orderByValue = _ref14.orderByValue,
      outerSkipLimit = _ref14.outerSkipLimit,
      nullParams = _ref14.nullParams,
      nonNullParams = _ref14.nonNullParams,
      filterParams = _ref14.filterParams,
      neo4jTypeArgs = _ref14.neo4jTypeArgs,
      _id = _ref14._id;
  var safeVariableName = (0, _utils.safeVar)(variableName);
  var safeLabelName = (0, _utils.safeLabel)([typeName].concat((0, _toConsumableArray2["default"])(additionalLabels)));
  var rootParamIndex = 1;

  var _buildCypherSelection3 = (0, _selections.buildCypherSelection)({
    cypherParams: cypherParams,
    selections: selections,
    variableName: variableName,
    schemaType: schemaType,
    resolveInfo: resolveInfo,
    paramIndex: rootParamIndex,
    isFederatedOperation: isFederatedOperation,
    context: context
  }),
      _buildCypherSelection4 = (0, _slicedToArray2["default"])(_buildCypherSelection3, 2),
      subQuery = _buildCypherSelection4[0],
      subParams = _buildCypherSelection4[1];

  var fieldArgs = (0, _utils.getQueryArguments)(resolveInfo, isFederatedOperation);

  var _processFilterArgumen7 = processFilterArgument({
    fieldArgs: fieldArgs,
    isFederatedOperation: isFederatedOperation,
    schemaType: schemaType,
    variableName: variableName,
    resolveInfo: resolveInfo,
    params: nonNullParams,
    paramIndex: rootParamIndex
  }),
      _processFilterArgumen8 = (0, _slicedToArray2["default"])(_processFilterArgumen7, 2),
      filterPredicates = _processFilterArgumen8[0],
      serializedFilter = _processFilterArgumen8[1];

  var params = _objectSpread({}, serializedFilter, {}, subParams);

  if (cypherParams) {
    params['cypherParams'] = cypherParams;
  }

  var arrayParams = _lodash["default"].pickBy(filterParams, _isArray["default"]);

  var args = (0, _utils.innerFilterParams)(filterParams, neo4jTypeArgs);
  var argString = (0, _utils.paramsToString)(_lodash["default"].filter(args, function (arg) {
    return !(0, _isArray["default"])(arg.value);
  }));
  var idWherePredicate = typeof _id !== 'undefined' ? "ID(".concat(safeVariableName, ")=").concat(_id) : '';
  var nullFieldPredicates = (0, _keys["default"])(nullParams).map(function (key) {
    return "".concat(variableName, ".").concat(key, " IS NULL");
  });

  var arrayPredicates = _lodash["default"].map(arrayParams, function (value, key) {
    return "".concat(safeVariableName, ".").concat((0, _utils.safeVar)(key), " IN $").concat(key);
  });

  var fragmentTypeParams = derivedTypesParams({
    isInterfaceType: isInterfaceType,
    isUnionType: isUnionType,
    schema: resolveInfo.schema,
    schemaTypeName: schemaType.name,
    usesFragments: usesFragments
  });

  var _buildMapProjection11 = buildMapProjection({
    schemaType: schemaType,
    schemaTypeFields: schemaTypeFields,
    derivedTypeMap: derivedTypeMap,
    isObjectType: isObjectType,
    isInterfaceType: isInterfaceType,
    isUnionType: isUnionType,
    usesFragments: usesFragments,
    safeVariableName: safeVariableName,
    subQuery: subQuery,
    resolveInfo: resolveInfo
  }),
      _buildMapProjection12 = (0, _slicedToArray2["default"])(_buildMapProjection11, 2),
      mapProjection = _buildMapProjection12[0],
      labelPredicate = _buildMapProjection12[1];

  var predicateClauses = [idWherePredicate, labelPredicate].concat((0, _toConsumableArray2["default"])(filterPredicates), (0, _toConsumableArray2["default"])(nullFieldPredicates), (0, _toConsumableArray2["default"])(neo4jTypeClauses), (0, _toConsumableArray2["default"])(arrayPredicates)).filter(function (predicate) {
    return !!predicate;
  }).join(' AND ');
  var predicate = predicateClauses ? "WHERE ".concat(predicateClauses, " ") : '';
  var optimization = orderByValue.optimization,
      orderByClause = orderByValue.cypherPart;
  var query = "MATCH (".concat(safeVariableName, ":").concat(safeLabelName).concat(argString ? " ".concat(argString) : '', ") ").concat(predicate).concat(optimization.earlyOrderBy ? "WITH ".concat(safeVariableName).concat(orderByClause) : '', "RETURN ").concat(mapProjection, " AS ").concat(safeVariableName).concat(optimization.earlyOrderBy ? '' : orderByClause).concat(outerSkipLimit);
  return [query, _objectSpread({}, params, {}, fragmentTypeParams)];
};

var buildMapProjection = function buildMapProjection(_ref15) {
  var schemaType = _ref15.schemaType,
      schemaTypeFields = _ref15.schemaTypeFields,
      listVariable = _ref15.listVariable,
      derivedTypeMap = _ref15.derivedTypeMap,
      isObjectType = _ref15.isObjectType,
      isInterfaceType = _ref15.isInterfaceType,
      isUnionType = _ref15.isUnionType,
      isScalarPayload = _ref15.isScalarPayload,
      isComputedQuery = _ref15.isComputedQuery,
      isComputedMutation = _ref15.isComputedMutation,
      isComputedField = _ref15.isComputedField,
      usesFragments = _ref15.usesFragments,
      safeVariableName = _ref15.safeVariableName,
      subQuery = _ref15.subQuery,
      resolveInfo = _ref15.resolveInfo;
  var labelPredicate = buildTypeCompositionPredicate({
    schemaType: schemaType,
    schemaTypeFields: schemaTypeFields,
    listVariable: listVariable,
    derivedTypeMap: derivedTypeMap,
    safeVariableName: safeVariableName,
    isInterfaceType: isInterfaceType,
    isUnionType: isUnionType,
    isComputedQuery: isComputedQuery,
    isComputedMutation: isComputedMutation,
    isComputedField: isComputedField,
    usesFragments: usesFragments,
    resolveInfo: resolveInfo
  });
  var isFragmentedInterfaceType = usesFragments && isInterfaceType;
  var isFragmentedUnionType = usesFragments && isUnionType;
  var mapProjection = '';

  if (isScalarPayload) {
    // A scalar type payload has no map projection
    mapProjection = safeVariableName;
  } else if (isObjectType) {
    mapProjection = "".concat(safeVariableName, " {").concat(subQuery, "}");
  } else if (isFragmentedInterfaceType || isFragmentedUnionType) {
    // An interface type possibly uses fragments and a
    // union type necessarily uses fragments
    mapProjection = subQuery;
  } else if (isInterfaceType || isUnionType) {
    // If no fragments are used, then this is an interface type
    // with only interface fields selected
    mapProjection = "".concat(safeVariableName, " {").concat(fragmentType(safeVariableName, schemaType.name)).concat(subQuery ? ",".concat(subQuery) : '', "}");
  }

  return [mapProjection, labelPredicate];
};

var getUnionLabels = function getUnionLabels(_ref16) {
  var _ref16$typeName = _ref16.typeName,
      typeName = _ref16$typeName === void 0 ? '' : _ref16$typeName,
      _ref16$typeMap = _ref16.typeMap,
      typeMap = _ref16$typeMap === void 0 ? {} : _ref16$typeMap;
  var unionLabels = [];
  (0, _keys["default"])(typeMap).map(function (key) {
    var definition = typeMap[key];
    var astNode = definition.astNode;

    if ((0, _types.isUnionTypeDefinition)({
      definition: astNode
    })) {
      var types = definition.getTypes();
      var unionTypeName = definition.name;

      if (types.find(function (type) {
        return type.name === typeName;
      })) {
        unionLabels.push(unionTypeName);
      }
    }
  });
  return unionLabels;
}; // Mutation API root operation branch


var translateMutation = function translateMutation(_ref17) {
  var resolveInfo = _ref17.resolveInfo,
      context = _ref17.context,
      first = _ref17.first,
      offset = _ref17.offset,
      otherParams = _ref17.otherParams;
  var typeMap = resolveInfo.schema.getTypeMap();

  var _typeIdentifiers2 = (0, _utils.typeIdentifiers)(resolveInfo.returnType),
      typeName = _typeIdentifiers2.typeName,
      variableName = _typeIdentifiers2.variableName;

  var schemaType = resolveInfo.schema.getType(typeName);
  var selections = (0, _utils.getPayloadSelections)(resolveInfo);
  var outerSkipLimit = (0, _utils.getOuterSkipLimit)(first, offset);
  var orderByValue = (0, _utils.computeOrderBy)(resolveInfo, schemaType);
  var additionalNodeLabels = (0, _utils.getAdditionalLabels)(schemaType, getCypherParams(context));
  var mutationTypeCypherDirective = (0, _utils.getMutationCypherDirective)(resolveInfo);
  var mutationMeta = resolveInfo.schema.getMutationType().getFields()[resolveInfo.fieldName].astNode.directives.find(function (x) {
    return x.name.value === 'MutationMeta';
  });
  var params = (0, _utils.initializeMutationParams)({
    mutationMeta: mutationMeta,
    resolveInfo: resolveInfo,
    mutationTypeCypherDirective: mutationTypeCypherDirective,
    first: first,
    otherParams: otherParams,
    offset: offset
  });
  var isInterfaceType = (0, _utils.isGraphqlInterfaceType)(schemaType);
  var isObjectType = (0, _utils.isGraphqlObjectType)(schemaType);
  var isUnionType = (0, _utils.isGraphqlUnionType)(schemaType);
  var usesFragments = (0, _selections.isFragmentedSelection)({
    selections: selections
  });
  var isFragmentedObjectType = isObjectType && usesFragments;
  var interfaceLabels = typeof schemaType.getInterfaces === 'function' ? schemaType.getInterfaces().map(function (i) {
    return i.name;
  }) : [];
  var unionLabels = getUnionLabels({
    typeName: typeName,
    typeMap: typeMap
  });
  var additionalLabels = [].concat((0, _toConsumableArray2["default"])(additionalNodeLabels), (0, _toConsumableArray2["default"])(interfaceLabels), (0, _toConsumableArray2["default"])(unionLabels));

  var _mergeSelectionFragme3 = (0, _selections.mergeSelectionFragments)({
    schemaType: schemaType,
    selections: selections,
    isFragmentedObjectType: isFragmentedObjectType,
    isUnionType: isUnionType,
    typeMap: typeMap,
    resolveInfo: resolveInfo
  }),
      _mergeSelectionFragme4 = (0, _slicedToArray2["default"])(_mergeSelectionFragme3, 2),
      schemaTypeFields = _mergeSelectionFragme4[0],
      derivedTypeMap = _mergeSelectionFragme4[1];

  if (mutationTypeCypherDirective) {
    return customMutation({
      resolveInfo: resolveInfo,
      schemaType: schemaType,
      schemaTypeFields: schemaTypeFields,
      derivedTypeMap: derivedTypeMap,
      isObjectType: isObjectType,
      isInterfaceType: isInterfaceType,
      isUnionType: isUnionType,
      usesFragments: usesFragments,
      selections: selections,
      params: params,
      context: context,
      mutationTypeCypherDirective: mutationTypeCypherDirective,
      variableName: variableName,
      orderByValue: orderByValue,
      outerSkipLimit: outerSkipLimit
    });
  } else if ((0, _utils.isCreateMutation)(resolveInfo)) {
    return nodeCreate({
      resolveInfo: resolveInfo,
      schemaType: schemaType,
      selections: selections,
      params: params,
      variableName: variableName,
      typeName: typeName,
      additionalLabels: additionalLabels
    });
  } else if ((0, _utils.isDeleteMutation)(resolveInfo)) {
    return nodeDelete({
      resolveInfo: resolveInfo,
      schemaType: schemaType,
      selections: selections,
      params: params,
      variableName: variableName,
      typeName: typeName
    });
  } else if ((0, _utils.isAddMutation)(resolveInfo)) {
    return relationshipCreate({
      resolveInfo: resolveInfo,
      schemaType: schemaType,
      selections: selections,
      params: params,
      context: context
    });
  } else if ((0, _utils.isUpdateMutation)(resolveInfo) || (0, _utils.isMergeMutation)(resolveInfo)) {
    /**
     * TODO: Once we are no longer using the @MutationMeta directive
     * on relationship mutations, we will need to more directly identify
     * whether this Merge mutation if for a node or relationship
     */
    if (mutationMeta) {
      return relationshipMergeOrUpdate({
        mutationMeta: mutationMeta,
        resolveInfo: resolveInfo,
        selections: selections,
        schemaType: schemaType,
        params: params,
        context: context
      });
    } else {
      return nodeMergeOrUpdate({
        resolveInfo: resolveInfo,
        variableName: variableName,
        typeName: typeName,
        selections: selections,
        schemaType: schemaType,
        additionalLabels: additionalLabels,
        params: params
      });
    }
  } else if ((0, _utils.isRemoveMutation)(resolveInfo)) {
    return relationshipDelete({
      resolveInfo: resolveInfo,
      schemaType: schemaType,
      selections: selections,
      params: params,
      context: context
    });
  } else {
    // throw error - don't know how to handle this type of mutation
    throw new Error('Do not know how to handle this type of mutation. Mutation does not follow naming convention.');
  }
}; // Custom write operation


exports.translateMutation = translateMutation;

var customMutation = function customMutation(_ref18) {
  var params = _ref18.params,
      context = _ref18.context,
      mutationTypeCypherDirective = _ref18.mutationTypeCypherDirective,
      selections = _ref18.selections,
      variableName = _ref18.variableName,
      schemaType = _ref18.schemaType,
      schemaTypeFields = _ref18.schemaTypeFields,
      derivedTypeMap = _ref18.derivedTypeMap,
      isObjectType = _ref18.isObjectType,
      isInterfaceType = _ref18.isInterfaceType,
      isUnionType = _ref18.isUnionType,
      usesFragments = _ref18.usesFragments,
      resolveInfo = _ref18.resolveInfo,
      orderByValue = _ref18.orderByValue,
      outerSkipLimit = _ref18.outerSkipLimit;
  var cypherParams = getCypherParams(context);
  var safeVariableName = (0, _utils.safeVar)(variableName); // FIXME: support IN for multiple values -> WHERE

  var argString = (0, _utils.paramsToString)((0, _utils.innerFilterParams)((0, _utils.getFilterParams)(params.params || params), null, null, true), cypherParams);
  var cypherQueryArg = mutationTypeCypherDirective.arguments.find(function (x) {
    return x.name.value === 'statement';
  });

  var _buildCypherSelection5 = (0, _selections.buildCypherSelection)({
    selections: selections,
    variableName: variableName,
    schemaType: schemaType,
    resolveInfo: resolveInfo,
    cypherParams: cypherParams
  }),
      _buildCypherSelection6 = (0, _slicedToArray2["default"])(_buildCypherSelection5, 2),
      subQuery = _buildCypherSelection6[0],
      subParams = _buildCypherSelection6[1];

  var isScalarType = (0, _utils.isGraphqlScalarType)(schemaType);
  var isNeo4jTypeOutput = (0, _utils.isNeo4jType)(schemaType.name);
  var isScalarField = isNeo4jTypeOutput || isScalarType;
  var orderByClause = orderByValue.cypherPart;
  var listVariable = "apoc.map.values(value, [keys(value)[0]])[0] ";

  var _buildMapProjection13 = buildMapProjection({
    isComputedMutation: true,
    listVariable: listVariable,
    schemaType: schemaType,
    schemaTypeFields: schemaTypeFields,
    derivedTypeMap: derivedTypeMap,
    isObjectType: isObjectType,
    isInterfaceType: isInterfaceType,
    isUnionType: isUnionType,
    usesFragments: usesFragments,
    safeVariableName: safeVariableName,
    subQuery: subQuery,
    resolveInfo: resolveInfo
  }),
      _buildMapProjection14 = (0, _slicedToArray2["default"])(_buildMapProjection13, 2),
      mapProjection = _buildMapProjection14[0],
      labelPredicate = _buildMapProjection14[1];

  var query = '';

  if (labelPredicate) {
    query = "CALL apoc.cypher.doIt(\"".concat(cypherQueryArg.value.value, "\", ").concat(argString, ") YIELD value\n    ").concat(!isScalarField ? labelPredicate : '', "AS ").concat(safeVariableName, "\n    RETURN ").concat(!isScalarField ? "".concat(mapProjection, " AS ").concat(safeVariableName).concat(orderByClause).concat(outerSkipLimit) : '');
  } else {
    query = "CALL apoc.cypher.doIt(\"".concat(cypherQueryArg.value.value, "\", ").concat(argString, ") YIELD value\n    WITH ").concat(listVariable, "AS ").concat(safeVariableName, "\n    RETURN ").concat(safeVariableName, " ").concat(!isScalarField ? "{".concat(isInterfaceType ? "".concat(fragmentType(safeVariableName, schemaType.name), ",") : '').concat(subQuery, "} AS ").concat(safeVariableName).concat(orderByClause).concat(outerSkipLimit) : '');
  }

  var fragmentTypeParams = derivedTypesParams({
    isInterfaceType: isInterfaceType,
    isUnionType: isUnionType,
    schema: resolveInfo.schema,
    schemaTypeName: schemaType.name,
    usesFragments: usesFragments
  });
  params = _objectSpread({}, params, {}, subParams, {}, fragmentTypeParams);

  if (cypherParams) {
    params['cypherParams'] = cypherParams;
  }

  return [query, _objectSpread({}, params)];
}; // Generated API
// Node Create - Update - Delete


var nodeCreate = function nodeCreate(_ref19) {
  var variableName = _ref19.variableName,
      typeName = _ref19.typeName,
      selections = _ref19.selections,
      schemaType = _ref19.schemaType,
      resolveInfo = _ref19.resolveInfo,
      additionalLabels = _ref19.additionalLabels,
      params = _ref19.params;
  var safeVariableName = (0, _utils.safeVar)(variableName);
  var safeLabelName = (0, _utils.safeLabel)([typeName].concat((0, _toConsumableArray2["default"])(additionalLabels)));
  var statements = [];
  var args = (0, _utils.getMutationArguments)(resolveInfo);
  statements = (0, _utils.possiblySetFirstId)({
    args: args,
    statements: statements,
    params: params.params
  });

  var _buildCypherParameter = (0, _utils.buildCypherParameters)({
    args: args,
    statements: statements,
    params: params,
    paramKey: 'params',
    resolveInfo: resolveInfo
  }),
      _buildCypherParameter2 = (0, _slicedToArray2["default"])(_buildCypherParameter, 2),
      preparedParams = _buildCypherParameter2[0],
      paramStatements = _buildCypherParameter2[1];

  var _buildCypherSelection7 = (0, _selections.buildCypherSelection)({
    selections: selections,
    variableName: variableName,
    schemaType: schemaType,
    resolveInfo: resolveInfo
  }),
      _buildCypherSelection8 = (0, _slicedToArray2["default"])(_buildCypherSelection7, 2),
      subQuery = _buildCypherSelection8[0],
      subParams = _buildCypherSelection8[1];

  params = _objectSpread({}, preparedParams, {}, subParams);
  var query = "\n    CREATE (".concat(safeVariableName, ":").concat(safeLabelName, " {").concat(paramStatements.join(','), "})\n    RETURN ").concat(safeVariableName, " {").concat(subQuery, "} AS ").concat(safeVariableName, "\n  ");
  return [query, params];
};

var nodeDelete = function nodeDelete(_ref20) {
  var resolveInfo = _ref20.resolveInfo,
      selections = _ref20.selections,
      variableName = _ref20.variableName,
      typeName = _ref20.typeName,
      schemaType = _ref20.schemaType,
      params = _ref20.params;
  var safeVariableName = (0, _utils.safeVar)(variableName);
  var safeLabelName = (0, _utils.safeLabel)(typeName);
  var args = (0, _utils.getMutationArguments)(resolveInfo);
  var primaryKeyArg = args[0];
  var primaryKeyArgName = primaryKeyArg.name.value;
  var neo4jTypeArgs = (0, _utils.getNeo4jTypeArguments)(args);

  var _splitSelectionParame = (0, _utils.splitSelectionParameters)(params, primaryKeyArgName),
      _splitSelectionParame2 = (0, _slicedToArray2["default"])(_splitSelectionParame, 1),
      primaryKeyParam = _splitSelectionParame2[0];

  var neo4jTypeClauses = (0, _utils.neo4jTypePredicateClauses)(primaryKeyParam, safeVariableName, neo4jTypeArgs);

  var _buildCypherParameter3 = (0, _utils.buildCypherParameters)({
    args: args,
    params: params,
    resolveInfo: resolveInfo
  }),
      _buildCypherParameter4 = (0, _slicedToArray2["default"])(_buildCypherParameter3, 1),
      preparedParams = _buildCypherParameter4[0];

  var query = "MATCH (".concat(safeVariableName, ":").concat(safeLabelName).concat(neo4jTypeClauses.length > 0 ? ") WHERE ".concat(neo4jTypeClauses.join(' AND ')) : " {".concat(primaryKeyArgName, ": $").concat(primaryKeyArgName, "})"));

  var _buildCypherSelection9 = (0, _selections.buildCypherSelection)({
    selections: selections,
    variableName: variableName,
    schemaType: schemaType,
    resolveInfo: resolveInfo
  }),
      _buildCypherSelection10 = (0, _slicedToArray2["default"])(_buildCypherSelection9, 2),
      subQuery = _buildCypherSelection10[0],
      subParams = _buildCypherSelection10[1];

  params = _objectSpread({}, preparedParams, {}, subParams);
  var deletionVariableName = (0, _utils.safeVar)("".concat(variableName, "_toDelete")); // Cannot execute a map projection on a deleted node in Neo4j
  // so the projection is executed and aliased before the delete

  query += "\nWITH ".concat(safeVariableName, " AS ").concat(deletionVariableName, ", ").concat(safeVariableName, " {").concat(subQuery, "} AS ").concat(safeVariableName, "\nDETACH DELETE ").concat(deletionVariableName, "\nRETURN ").concat(safeVariableName);
  return [query, params];
}; // Relation Add / Remove


var relationshipCreate = function relationshipCreate(_ref21) {
  var resolveInfo = _ref21.resolveInfo,
      selections = _ref21.selections,
      schemaType = _ref21.schemaType,
      params = _ref21.params,
      context = _ref21.context;
  var mutationMeta, relationshipNameArg, fromTypeArg, toTypeArg;

  try {
    mutationMeta = resolveInfo.schema.getMutationType().getFields()[resolveInfo.fieldName].astNode.directives.find(function (x) {
      return x.name.value === 'MutationMeta';
    });
  } catch (e) {
    throw new Error('Missing required MutationMeta directive on add relationship directive');
  }

  try {
    relationshipNameArg = mutationMeta.arguments.find(function (x) {
      return x.name.value === 'relationship';
    });
    fromTypeArg = mutationMeta.arguments.find(function (x) {
      return x.name.value === 'from';
    });
    toTypeArg = mutationMeta.arguments.find(function (x) {
      return x.name.value === 'to';
    });
  } catch (e) {
    throw new Error('Missing required argument in MutationMeta directive (relationship, from, or to)');
  } //TODO: need to handle one-to-one and one-to-many


  var args = (0, _utils.getMutationArguments)(resolveInfo);
  var typeMap = resolveInfo.schema.getTypeMap();
  var cypherParams = getCypherParams(context);
  var fromType = fromTypeArg.value.value;
  var fromVar = "".concat((0, _utils.lowFirstLetter)(fromType), "_from");
  var fromInputArg = args.find(function (e) {
    return e.name.value === 'from';
  }).type;
  var fromInputAst = typeMap[(0, _graphql.getNamedType)(fromInputArg).type.name.value].astNode;
  var fromFields = fromInputAst.fields;
  var fromParam = fromFields[0].name.value;
  var fromNodeNeo4jTypeArgs = (0, _utils.getNeo4jTypeArguments)(fromFields);
  var toType = toTypeArg.value.value;
  var toVar = "".concat((0, _utils.lowFirstLetter)(toType), "_to");
  var toInputArg = args.find(function (e) {
    return e.name.value === 'to';
  }).type;
  var toInputAst = typeMap[(0, _graphql.getNamedType)(toInputArg).type.name.value].astNode;
  var toFields = toInputAst.fields;
  var toParam = toFields[0].name.value;
  var toNodeNeo4jTypeArgs = (0, _utils.getNeo4jTypeArguments)(toFields);
  var relationshipName = relationshipNameArg.value.value;
  var lowercased = relationshipName.toLowerCase();
  var dataInputArg = args.find(function (e) {
    return e.name.value === 'data';
  });
  var dataInputAst = dataInputArg ? typeMap[(0, _graphql.getNamedType)(dataInputArg.type).type.name.value].astNode : undefined;
  var dataFields = dataInputAst ? dataInputAst.fields : [];

  var _buildCypherParameter5 = (0, _utils.buildCypherParameters)({
    args: dataFields,
    params: params,
    paramKey: 'data',
    resolveInfo: resolveInfo
  }),
      _buildCypherParameter6 = (0, _slicedToArray2["default"])(_buildCypherParameter5, 2),
      preparedParams = _buildCypherParameter6[0],
      paramStatements = _buildCypherParameter6[1];

  var schemaTypeName = (0, _utils.safeVar)(schemaType);
  var fromVariable = (0, _utils.safeVar)(fromVar);
  var fromAdditionalLabels = (0, _utils.getAdditionalLabels)(resolveInfo.schema.getType(fromType), cypherParams);
  var fromLabel = (0, _utils.safeLabel)([fromType].concat((0, _toConsumableArray2["default"])(fromAdditionalLabels)));
  var toVariable = (0, _utils.safeVar)(toVar);
  var toAdditionalLabels = (0, _utils.getAdditionalLabels)(resolveInfo.schema.getType(toType), cypherParams);
  var toLabel = (0, _utils.safeLabel)([toType].concat((0, _toConsumableArray2["default"])(toAdditionalLabels)));
  var relationshipVariable = (0, _utils.safeVar)(lowercased + '_relation');
  var relationshipLabel = (0, _utils.safeLabel)(relationshipName);
  var fromNodeNeo4jTypeClauses = (0, _utils.neo4jTypePredicateClauses)(preparedParams.from, fromVariable, fromNodeNeo4jTypeArgs, 'from');
  var toNodeNeo4jTypeClauses = (0, _utils.neo4jTypePredicateClauses)(preparedParams.to, toVariable, toNodeNeo4jTypeArgs, 'to');

  var _buildCypherSelection11 = (0, _selections.buildCypherSelection)({
    selections: selections,
    schemaType: schemaType,
    resolveInfo: resolveInfo,
    parentSelectionInfo: {
      rootType: 'relationship',
      from: fromVar,
      to: toVar,
      variableName: lowercased
    },
    cypherParams: getCypherParams(context)
  }),
      _buildCypherSelection12 = (0, _slicedToArray2["default"])(_buildCypherSelection11, 2),
      subQuery = _buildCypherSelection12[0],
      subParams = _buildCypherSelection12[1];

  params = _objectSpread({}, preparedParams, {}, subParams);
  var query = "\n      MATCH (".concat(fromVariable, ":").concat(fromLabel).concat(fromNodeNeo4jTypeClauses && fromNodeNeo4jTypeClauses.length > 0 ? // uses either a WHERE clause for managed type primary keys (temporal, etc.)
  ") WHERE ".concat(fromNodeNeo4jTypeClauses.join(' AND '), " ") : // or a an internal matching clause for normal, scalar property primary keys
  // NOTE this will need to change if we at some point allow for multi field node selection
  " {".concat(fromParam, ": $from.").concat(fromParam, "})"), "\n      MATCH (").concat(toVariable, ":").concat(toLabel).concat(toNodeNeo4jTypeClauses && toNodeNeo4jTypeClauses.length > 0 ? ") WHERE ".concat(toNodeNeo4jTypeClauses.join(' AND '), " ") : " {".concat(toParam, ": $to.").concat(toParam, "})"), "\n      CREATE (").concat(fromVariable, ")-[").concat(relationshipVariable, ":").concat(relationshipLabel).concat(paramStatements.length > 0 ? " {".concat(paramStatements.join(','), "}") : '', "]->(").concat(toVariable, ")\n      RETURN ").concat(relationshipVariable, " { ").concat(subQuery, " } AS ").concat(schemaTypeName, ";\n    ");
  return [query, params];
};

var relationshipDelete = function relationshipDelete(_ref22) {
  var resolveInfo = _ref22.resolveInfo,
      selections = _ref22.selections,
      schemaType = _ref22.schemaType,
      params = _ref22.params,
      context = _ref22.context;
  var mutationMeta, relationshipNameArg, fromTypeArg, toTypeArg;

  try {
    mutationMeta = resolveInfo.schema.getMutationType().getFields()[resolveInfo.fieldName].astNode.directives.find(function (x) {
      return x.name.value === 'MutationMeta';
    });
  } catch (e) {
    throw new Error('Missing required MutationMeta directive on add relationship directive');
  }

  try {
    relationshipNameArg = mutationMeta.arguments.find(function (x) {
      return x.name.value === 'relationship';
    });
    fromTypeArg = mutationMeta.arguments.find(function (x) {
      return x.name.value === 'from';
    });
    toTypeArg = mutationMeta.arguments.find(function (x) {
      return x.name.value === 'to';
    });
  } catch (e) {
    throw new Error('Missing required argument in MutationMeta directive (relationship, from, or to)');
  } //TODO: need to handle one-to-one and one-to-many


  var args = (0, _utils.getMutationArguments)(resolveInfo);
  var typeMap = resolveInfo.schema.getTypeMap();
  var cypherParams = getCypherParams(context);
  var fromType = fromTypeArg.value.value;
  var fromVar = "".concat((0, _utils.lowFirstLetter)(fromType), "_from");
  var fromInputArg = args.find(function (e) {
    return e.name.value === 'from';
  }).type;
  var fromInputAst = typeMap[(0, _graphql.getNamedType)(fromInputArg).type.name.value].astNode;
  var fromFields = fromInputAst.fields;
  var fromParam = fromFields[0].name.value;
  var fromNodeNeo4jTypeArgs = (0, _utils.getNeo4jTypeArguments)(fromFields);
  var toType = toTypeArg.value.value;
  var toVar = "".concat((0, _utils.lowFirstLetter)(toType), "_to");
  var toInputArg = args.find(function (e) {
    return e.name.value === 'to';
  }).type;
  var toInputAst = typeMap[(0, _graphql.getNamedType)(toInputArg).type.name.value].astNode;
  var toFields = toInputAst.fields;
  var toParam = toFields[0].name.value;
  var toNodeNeo4jTypeArgs = (0, _utils.getNeo4jTypeArguments)(toFields);
  var relationshipName = relationshipNameArg.value.value;
  var schemaTypeName = (0, _utils.safeVar)(schemaType);
  var fromVariable = (0, _utils.safeVar)(fromVar);
  var fromAdditionalLabels = (0, _utils.getAdditionalLabels)(resolveInfo.schema.getType(fromType), cypherParams);
  var fromLabel = (0, _utils.safeLabel)([fromType].concat((0, _toConsumableArray2["default"])(fromAdditionalLabels)));
  var toVariable = (0, _utils.safeVar)(toVar);
  var toAdditionalLabels = (0, _utils.getAdditionalLabels)(resolveInfo.schema.getType(toType), cypherParams);
  var toLabel = (0, _utils.safeLabel)([toType].concat((0, _toConsumableArray2["default"])(toAdditionalLabels)));
  var relationshipVariable = (0, _utils.safeVar)(fromVar + toVar);
  var relationshipLabel = (0, _utils.safeLabel)(relationshipName);
  var fromRootVariable = (0, _utils.safeVar)('_' + fromVar);
  var toRootVariable = (0, _utils.safeVar)('_' + toVar);
  var fromNodeNeo4jTypeClauses = (0, _utils.neo4jTypePredicateClauses)(params.from, fromVariable, fromNodeNeo4jTypeArgs, 'from');
  var toNodeNeo4jTypeClauses = (0, _utils.neo4jTypePredicateClauses)(params.to, toVariable, toNodeNeo4jTypeArgs, 'to'); // TODO cleaner semantics: remove use of _ prefixes in root variableNames and variableName

  var _buildCypherSelection13 = (0, _selections.buildCypherSelection)({
    selections: selections,
    schemaType: schemaType,
    resolveInfo: resolveInfo,
    parentSelectionInfo: {
      rootType: 'relationship',
      from: "_".concat(fromVar),
      to: "_".concat(toVar)
    },
    cypherParams: getCypherParams(context)
  }),
      _buildCypherSelection14 = (0, _slicedToArray2["default"])(_buildCypherSelection13, 2),
      subQuery = _buildCypherSelection14[0],
      subParams = _buildCypherSelection14[1];

  params = _objectSpread({}, params, {}, subParams);
  var query = "\n      MATCH (".concat(fromVariable, ":").concat(fromLabel).concat(fromNodeNeo4jTypeClauses && fromNodeNeo4jTypeClauses.length > 0 ? // uses either a WHERE clause for managed type primary keys (temporal, etc.)
  ") WHERE ".concat(fromNodeNeo4jTypeClauses.join(' AND '), " ") : // or a an internal matching clause for normal, scalar property primary keys
  " {".concat(fromParam, ": $from.").concat(fromParam, "})"), "\n      MATCH (").concat(toVariable, ":").concat(toLabel).concat(toNodeNeo4jTypeClauses && toNodeNeo4jTypeClauses.length > 0 ? ") WHERE ".concat(toNodeNeo4jTypeClauses.join(' AND '), " ") : " {".concat(toParam, ": $to.").concat(toParam, "})"), "\n      OPTIONAL MATCH (").concat(fromVariable, ")-[").concat(relationshipVariable, ":").concat(relationshipLabel, "]->(").concat(toVariable, ")\n      DELETE ").concat(relationshipVariable, "\n      WITH COUNT(*) AS scope, ").concat(fromVariable, " AS ").concat(fromRootVariable, ", ").concat(toVariable, " AS ").concat(toRootVariable, "\n      RETURN {").concat(subQuery, "} AS ").concat(schemaTypeName, ";\n    ");
  return [query, params];
};

var relationshipMergeOrUpdate = function relationshipMergeOrUpdate(_ref23) {
  var mutationMeta = _ref23.mutationMeta,
      resolveInfo = _ref23.resolveInfo,
      selections = _ref23.selections,
      schemaType = _ref23.schemaType,
      params = _ref23.params,
      context = _ref23.context;
  var query = '';
  var relationshipNameArg = undefined;
  var fromTypeArg = undefined;
  var toTypeArg = undefined;

  try {
    relationshipNameArg = mutationMeta.arguments.find(function (x) {
      return x.name.value === 'relationship';
    });
    fromTypeArg = mutationMeta.arguments.find(function (x) {
      return x.name.value === 'from';
    });
    toTypeArg = mutationMeta.arguments.find(function (x) {
      return x.name.value === 'to';
    });
  } catch (e) {
    throw new Error('Missing required argument in MutationMeta directive (relationship, from, or to)');
  }

  if (relationshipNameArg && fromTypeArg && toTypeArg) {
    //TODO: need to handle one-to-one and one-to-many
    var args = (0, _utils.getMutationArguments)(resolveInfo);
    var typeMap = resolveInfo.schema.getTypeMap();
    var cypherParams = getCypherParams(context);
    var fromType = fromTypeArg.value.value;
    var fromVar = "".concat((0, _utils.lowFirstLetter)(fromType), "_from");
    var fromInputArg = args.find(function (e) {
      return e.name.value === 'from';
    }).type;
    var fromInputAst = typeMap[(0, _graphql.getNamedType)(fromInputArg).type.name.value].astNode;
    var fromFields = fromInputAst.fields;
    var fromParam = fromFields[0].name.value;
    var fromNodeNeo4jTypeArgs = (0, _utils.getNeo4jTypeArguments)(fromFields);
    var toType = toTypeArg.value.value;
    var toVar = "".concat((0, _utils.lowFirstLetter)(toType), "_to");
    var toInputArg = args.find(function (e) {
      return e.name.value === 'to';
    }).type;
    var toInputAst = typeMap[(0, _graphql.getNamedType)(toInputArg).type.name.value].astNode;
    var toFields = toInputAst.fields;
    var toParam = toFields[0].name.value;
    var toNodeNeo4jTypeArgs = (0, _utils.getNeo4jTypeArguments)(toFields);
    var relationshipName = relationshipNameArg.value.value;
    var lowercased = relationshipName.toLowerCase();
    var dataInputArg = args.find(function (e) {
      return e.name.value === 'data';
    });
    var dataInputAst = dataInputArg ? typeMap[(0, _graphql.getNamedType)(dataInputArg.type).type.name.value].astNode : undefined;
    var dataFields = dataInputAst ? dataInputAst.fields : [];

    var _buildCypherParameter7 = (0, _utils.buildCypherParameters)({
      args: dataFields,
      params: params,
      paramKey: 'data',
      resolveInfo: resolveInfo
    }),
        _buildCypherParameter8 = (0, _slicedToArray2["default"])(_buildCypherParameter7, 2),
        preparedParams = _buildCypherParameter8[0],
        paramStatements = _buildCypherParameter8[1];

    var schemaTypeName = (0, _utils.safeVar)(schemaType);
    var fromVariable = (0, _utils.safeVar)(fromVar);
    var fromAdditionalLabels = (0, _utils.getAdditionalLabels)(resolveInfo.schema.getType(fromType), cypherParams);
    var fromLabel = (0, _utils.safeLabel)([fromType].concat((0, _toConsumableArray2["default"])(fromAdditionalLabels)));
    var toVariable = (0, _utils.safeVar)(toVar);
    var toAdditionalLabels = (0, _utils.getAdditionalLabels)(resolveInfo.schema.getType(toType), cypherParams);
    var toLabel = (0, _utils.safeLabel)([toType].concat((0, _toConsumableArray2["default"])(toAdditionalLabels)));
    var relationshipVariable = (0, _utils.safeVar)(lowercased + '_relation');
    var relationshipLabel = (0, _utils.safeLabel)(relationshipName);
    var fromNodeNeo4jTypeClauses = (0, _utils.neo4jTypePredicateClauses)(preparedParams.from, fromVariable, fromNodeNeo4jTypeArgs, 'from');
    var toNodeNeo4jTypeClauses = (0, _utils.neo4jTypePredicateClauses)(preparedParams.to, toVariable, toNodeNeo4jTypeArgs, 'to');

    var _buildCypherSelection15 = (0, _selections.buildCypherSelection)({
      selections: selections,
      schemaType: schemaType,
      resolveInfo: resolveInfo,
      parentSelectionInfo: {
        rootType: 'relationship',
        from: fromVar,
        to: toVar,
        variableName: lowercased
      },
      cypherParams: getCypherParams(context)
    }),
        _buildCypherSelection16 = (0, _slicedToArray2["default"])(_buildCypherSelection15, 2),
        subQuery = _buildCypherSelection16[0],
        subParams = _buildCypherSelection16[1];

    var cypherOperation = '';

    if ((0, _utils.isMergeMutation)(resolveInfo)) {
      cypherOperation = 'MERGE';
    } else if ((0, _utils.isUpdateMutation)(resolveInfo)) {
      cypherOperation = 'MATCH';
    }

    params = _objectSpread({}, preparedParams, {}, subParams);
    query = "\n      MATCH (".concat(fromVariable, ":").concat(fromLabel).concat(fromNodeNeo4jTypeClauses && fromNodeNeo4jTypeClauses.length > 0 ? // uses either a WHERE clause for managed type primary keys (temporal, etc.)
    ") WHERE ".concat(fromNodeNeo4jTypeClauses.join(' AND '), " ") : // or a an internal matching clause for normal, scalar property primary keys
    // NOTE this will need to change if we at some point allow for multi field node selection
    " {".concat(fromParam, ": $from.").concat(fromParam, "})"), "\n      MATCH (").concat(toVariable, ":").concat(toLabel).concat(toNodeNeo4jTypeClauses && toNodeNeo4jTypeClauses.length > 0 ? ") WHERE ".concat(toNodeNeo4jTypeClauses.join(' AND '), " ") : " {".concat(toParam, ": $to.").concat(toParam, "})"), "\n      ").concat(cypherOperation, " (").concat(fromVariable, ")-[").concat(relationshipVariable, ":").concat(relationshipLabel, "]->(").concat(toVariable, ")").concat(paramStatements.length > 0 ? "\n      SET ".concat(relationshipVariable, " += {").concat(paramStatements.join(','), "} ") : '', "\n      RETURN ").concat(relationshipVariable, " { ").concat(subQuery, " } AS ").concat(schemaTypeName, ";\n    ");
  }

  return [query, params];
};

var nodeMergeOrUpdate = function nodeMergeOrUpdate(_ref24) {
  var resolveInfo = _ref24.resolveInfo,
      variableName = _ref24.variableName,
      typeName = _ref24.typeName,
      selections = _ref24.selections,
      schemaType = _ref24.schemaType,
      additionalLabels = _ref24.additionalLabels,
      params = _ref24.params;
  var safeVariableName = (0, _utils.safeVar)(variableName);
  var args = (0, _utils.getMutationArguments)(resolveInfo);
  var primaryKeyArg = args[0];
  var primaryKeyArgName = primaryKeyArg.name.value;
  var neo4jTypeArgs = (0, _utils.getNeo4jTypeArguments)(args);

  var _splitSelectionParame3 = (0, _utils.splitSelectionParameters)(params, primaryKeyArgName, 'params'),
      _splitSelectionParame4 = (0, _slicedToArray2["default"])(_splitSelectionParame3, 2),
      primaryKeyParam = _splitSelectionParame4[0],
      updateParams = _splitSelectionParame4[1];

  var neo4jTypeClauses = (0, _utils.neo4jTypePredicateClauses)(primaryKeyParam, safeVariableName, neo4jTypeArgs, 'params');
  var predicateClauses = (0, _toConsumableArray2["default"])(neo4jTypeClauses).filter(function (predicate) {
    return !!predicate;
  }).join(' AND ');
  var predicate = predicateClauses ? "WHERE ".concat(predicateClauses, " ") : '';

  var _buildCypherParameter9 = (0, _utils.buildCypherParameters)({
    args: args,
    params: updateParams,
    paramKey: 'params',
    resolveInfo: resolveInfo
  }),
      _buildCypherParameter10 = (0, _slicedToArray2["default"])(_buildCypherParameter9, 2),
      preparedParams = _buildCypherParameter10[0],
      paramUpdateStatements = _buildCypherParameter10[1];

  var cypherOperation = '';
  var safeLabelName = (0, _utils.safeLabel)(typeName);

  if ((0, _utils.isMergeMutation)(resolveInfo)) {
    safeLabelName = (0, _utils.safeLabel)([typeName].concat((0, _toConsumableArray2["default"])(additionalLabels)));
    cypherOperation = 'MERGE';
  } else if ((0, _utils.isUpdateMutation)(resolveInfo)) {
    cypherOperation = 'MATCH';
  }

  var query = "".concat(cypherOperation, " (").concat(safeVariableName, ":").concat(safeLabelName).concat(predicate !== '' ? ") ".concat(predicate, " ") : "{".concat(primaryKeyArgName, ": $params.").concat(primaryKeyArgName, "})"), "\n  ");

  if (paramUpdateStatements.length > 0) {
    query += "SET ".concat(safeVariableName, " += {").concat(paramUpdateStatements.join(','), "} ");
  }

  var _buildCypherSelection17 = (0, _selections.buildCypherSelection)({
    selections: selections,
    variableName: variableName,
    schemaType: schemaType,
    resolveInfo: resolveInfo
  }),
      _buildCypherSelection18 = (0, _slicedToArray2["default"])(_buildCypherSelection17, 2),
      subQuery = _buildCypherSelection18[0],
      subParams = _buildCypherSelection18[1];

  preparedParams.params[primaryKeyArgName] = primaryKeyParam[primaryKeyArgName];
  params = _objectSpread({}, preparedParams, {}, subParams);
  query += "RETURN ".concat(safeVariableName, " {").concat(subQuery, "} AS ").concat(safeVariableName);
  return [query, params];
};

var buildOrderedNeo4jTypeSelections = function buildOrderedNeo4jTypeSelections(_ref25) {
  var schemaType = _ref25.schemaType,
      selections = _ref25.selections,
      usesTemporalOrdering = _ref25.usesTemporalOrdering,
      selectedFieldNames = _ref25.selectedFieldNames;
  var neo4jTypeSelections = '';

  if (usesTemporalOrdering) {
    neo4jTypeSelections = selections.reduce(function (temporalTypeFields, innerSelection) {
      // name of temporal type field
      var fieldName = innerSelection.name.value;
      var fieldTypeName = getFieldTypeName(schemaType, fieldName);
      var fieldIsSelected = selectedFieldNames.some(function (name) {
        return name === fieldName;
      });
      var isTemporalTypeField = (0, _utils.isTemporalType)(fieldTypeName);

      if (isTemporalTypeField && fieldIsSelected) {
        var innerSelectedTypes = innerSelection.selectionSet ? innerSelection.selectionSet.selections : [];
        temporalTypeFields.push("".concat(fieldName, ": {").concat(innerSelectedTypes.reduce(function (temporalSubFields, t) {
          // temporal type subfields, year, minute, etc.
          var subFieldName = t.name.value;

          if (subFieldName === 'formatted') {
            temporalSubFields.push("".concat(subFieldName, ": toString(sortedElement.").concat(fieldName, ")"));
          } else {
            temporalSubFields.push("".concat(subFieldName, ": sortedElement.").concat(fieldName, ".").concat(subFieldName));
          }

          return temporalSubFields;
        }, []).join(','), "}"));
      }

      return temporalTypeFields;
    }, []).join(',');
  }

  return neo4jTypeSelections;
};

var getFieldTypeName = function getFieldTypeName(schemaType, fieldName) {
  var field = schemaType && fieldName ? schemaType.getFields()[fieldName] : undefined;
  return field ? field.type.name : '';
};

var temporalOrderingFieldExists = function temporalOrderingFieldExists(schemaType, filterParams) {
  var orderByParam = filterParams ? filterParams['orderBy'] : undefined;

  if (orderByParam) {
    orderByParam = orderByParam.value;
    if (!(0, _isArray["default"])(orderByParam)) orderByParam = [orderByParam];
    return orderByParam.find(function (e) {
      var fieldName = e.substring(0, e.lastIndexOf('_'));
      var fieldTypeName = getFieldTypeName(schemaType, fieldName);
      return (0, _utils.isTemporalType)(fieldTypeName);
    });
  }

  return undefined;
};

var buildSortMultiArgs = function buildSortMultiArgs(param) {
  var values = param ? param.value : [];
  var fieldName = '';
  if (!(0, _isArray["default"])(values)) values = [values];
  return values.map(function (e) {
    fieldName = e.substring(0, e.lastIndexOf('_'));
    return e.includes('_asc') ? "'^".concat(fieldName, "'") : "'".concat(fieldName, "'");
  }).join(',');
};

var processFilterArgument = function processFilterArgument(_ref26) {
  var fieldArgs = _ref26.fieldArgs,
      isFederatedOperation = _ref26.isFederatedOperation,
      schemaType = _ref26.schemaType,
      variableName = _ref26.variableName,
      resolveInfo = _ref26.resolveInfo,
      params = _ref26.params,
      paramIndex = _ref26.paramIndex,
      parentSchemaType = _ref26.parentSchemaType,
      _ref26$rootIsRelation = _ref26.rootIsRelationType,
      rootIsRelationType = _ref26$rootIsRelation === void 0 ? false : _ref26$rootIsRelation;
  var filterArg = fieldArgs.find(function (e) {
    return e.name.value === 'filter';
  });
  var filterValue = (0, _keys["default"])(params).length ? params['filter'] : undefined;
  var filterParamKey = paramIndex > 1 ? "".concat(paramIndex - 1, "_filter") : "filter";
  var filterCypherParam = "$".concat(filterParamKey);
  var translations = []; // allows an exception for the existence of the filter argument AST
  // if isFederatedOperation

  if ((filterArg || isFederatedOperation) && filterValue) {
    // if field has both a filter argument and argument data is provided
    var schema = resolveInfo.schema;
    var serializedFilterParam = filterValue;
    var filterFieldMap = {};

    var _analyzeFilterArgumen = analyzeFilterArguments({
      filterValue: filterValue,
      variableName: variableName,
      filterCypherParam: filterCypherParam,
      schemaType: schemaType,
      schema: schema
    });

    var _analyzeFilterArgumen2 = (0, _slicedToArray2["default"])(_analyzeFilterArgumen, 2);

    filterFieldMap = _analyzeFilterArgumen2[0];
    serializedFilterParam = _analyzeFilterArgumen2[1];
    translations = translateFilterArguments({
      filterFieldMap: filterFieldMap,
      filterCypherParam: filterCypherParam,
      rootIsRelationType: rootIsRelationType,
      variableName: variableName,
      schemaType: schemaType,
      parentSchemaType: parentSchemaType,
      schema: schema
    });
    params = _objectSpread({}, params, (0, _defineProperty3["default"])({}, filterParamKey, serializedFilterParam));
  }

  return [translations, params];
};

var analyzeFilterArguments = function analyzeFilterArguments(_ref27) {
  var filterValue = _ref27.filterValue,
      variableName = _ref27.variableName,
      filterCypherParam = _ref27.filterCypherParam,
      schemaType = _ref27.schemaType,
      schema = _ref27.schema;
  return (0, _entries["default"])(filterValue).reduce(function (_ref28, _ref29) {
    var _ref30 = (0, _slicedToArray2["default"])(_ref28, 2),
        filterFieldMap = _ref30[0],
        serializedParams = _ref30[1];

    var _ref31 = (0, _slicedToArray2["default"])(_ref29, 2),
        name = _ref31[0],
        value = _ref31[1];

    var filterParamName = serializeFilterFieldName(name, value);

    var _analyzeFilterArgumen3 = analyzeFilterArgument({
      filterValue: value,
      filterValues: filterValue,
      fieldName: name,
      filterParam: filterCypherParam,
      variableName: variableName,
      schemaType: schemaType,
      schema: schema
    }),
        _analyzeFilterArgumen4 = (0, _slicedToArray2["default"])(_analyzeFilterArgumen3, 2),
        serializedValue = _analyzeFilterArgumen4[0],
        fieldMap = _analyzeFilterArgumen4[1];

    filterFieldMap[filterParamName] = fieldMap;
    serializedParams[filterParamName] = serializedValue;
    return [filterFieldMap, serializedParams];
  }, [{}, {}]);
};

var analyzeFilterArgument = function analyzeFilterArgument(_ref32) {
  var parentFieldName = _ref32.parentFieldName,
      filterValue = _ref32.filterValue,
      fieldName = _ref32.fieldName,
      variableName = _ref32.variableName,
      filterParam = _ref32.filterParam,
      parentSchemaType = _ref32.parentSchemaType,
      schemaType = _ref32.schemaType,
      schema = _ref32.schema;
  var parsedFilterName = parseFilterArgumentName(fieldName);
  var filterOperationField = parsedFilterName.name;
  var filterOperationType = parsedFilterName.type; // defaults

  var filterMapValue = true;
  var serializedFilterParam = filterValue;
  var innerSchemaType = schemaType;
  var typeName = schemaType.name;

  if (filterOperationField !== 'OR' && filterOperationField !== 'AND') {
    var schemaTypeFields = schemaType.getFields();
    var filterField = schemaTypeFields[filterOperationField];
    var filterFieldAst = filterField.astNode;
    var filterType = filterFieldAst.type;
    var innerFieldType = (0, _fields.unwrapNamedType)({
      type: filterType
    });
    typeName = innerFieldType.name;
    innerSchemaType = schema.getType(typeName);
  }

  if ((0, _graphql.isScalarType)(innerSchemaType) || (0, _graphql.isEnumType)(innerSchemaType)) {
    if (isExistentialFilter(filterOperationType, filterValue)) {
      serializedFilterParam = true;
      filterMapValue = null;
    }
  } else if ((0, _graphql.isObjectType)(innerSchemaType) || (0, _graphql.isInterfaceType)(innerSchemaType)) {
    if (fieldName === 'AND' || fieldName === 'OR') {
      // recursion
      var _analyzeNestedFilterA = analyzeNestedFilterArgument({
        filterValue: filterValue,
        filterOperationType: filterOperationType,
        parentFieldName: fieldName,
        parentSchemaType: schemaType,
        schemaType: schemaType,
        variableName: variableName,
        filterParam: filterParam,
        schema: schema
      });

      var _analyzeNestedFilterA2 = (0, _slicedToArray2["default"])(_analyzeNestedFilterA, 2);

      serializedFilterParam = _analyzeNestedFilterA2[0];
      filterMapValue = _analyzeNestedFilterA2[1];
    } else {
      var schemaTypeField = schemaType.getFields()[filterOperationField];

      var _innerSchemaType = (0, _utils.innerType)(schemaTypeField.type);

      var isObjectTypeFilter = (0, _graphql.isObjectType)(_innerSchemaType);
      var isInterfaceTypeFilter = (0, _graphql.isInterfaceType)(_innerSchemaType);

      if (isObjectTypeFilter || isInterfaceTypeFilter) {
        var _decideRelationFilter = decideRelationFilterMetadata({
          fieldName: fieldName,
          parentSchemaType: parentSchemaType,
          schemaType: schemaType,
          variableName: variableName,
          innerSchemaType: _innerSchemaType,
          filterOperationField: filterOperationField
        }),
            _decideRelationFilter2 = (0, _slicedToArray2["default"])(_decideRelationFilter, 9),
            thisType = _decideRelationFilter2[0],
            relatedType = _decideRelationFilter2[1],
            relationLabel = _decideRelationFilter2[2],
            relationDirection = _decideRelationFilter2[3],
            isRelation = _decideRelationFilter2[4],
            isRelationType = _decideRelationFilter2[5],
            isRelationTypeNode = _decideRelationFilter2[6],
            isReflexiveRelationType = _decideRelationFilter2[7],
            isReflexiveTypeDirectedField = _decideRelationFilter2[8];

        if (isReflexiveTypeDirectedField) {
          // for the 'from' and 'to' fields on the payload of a reflexive
          // relation type to use the parent field name, ex: 'knows_some'
          // is used for 'from' and 'to' in 'knows_some: { from: {}, to: {} }'
          var _parsedFilterName = parseFilterArgumentName(parentFieldName);

          filterOperationField = _parsedFilterName.name;
          filterOperationType = _parsedFilterName.type;
        }

        if (isExistentialFilter(filterOperationType, filterValue)) {
          serializedFilterParam = true;
          filterMapValue = null;
        } else if ((0, _utils.isTemporalType)(typeName) || (0, _utils.isSpatialType)(typeName) || (0, _utils.isSpatialDistanceInputType)({
          filterOperationType: filterOperationType
        })) {
          var _analyzeNeo4jTypeFilt = analyzeNeo4jTypeFilter({
            filterOperationType: filterOperationType,
            filterValue: filterValue,
            parentFieldName: parentFieldName
          });

          var _analyzeNeo4jTypeFilt2 = (0, _slicedToArray2["default"])(_analyzeNeo4jTypeFilt, 2);

          serializedFilterParam = _analyzeNeo4jTypeFilt2[0];
          filterMapValue = _analyzeNeo4jTypeFilt2[1];
        } else if (isRelation || isRelationType || isRelationTypeNode) {
          // recursion
          var _analyzeNestedFilterA3 = analyzeNestedFilterArgument({
            filterValue: filterValue,
            filterOperationType: filterOperationType,
            isRelationType: isRelationType,
            parentFieldName: fieldName,
            parentSchemaType: schemaType,
            schemaType: _innerSchemaType,
            variableName: variableName,
            filterParam: filterParam,
            schema: schema
          });

          var _analyzeNestedFilterA4 = (0, _slicedToArray2["default"])(_analyzeNestedFilterA3, 2);

          serializedFilterParam = _analyzeNestedFilterA4[0];
          filterMapValue = _analyzeNestedFilterA4[1];
        }
      }
    }
  }

  return [serializedFilterParam, filterMapValue];
};

var analyzeNeo4jTypeFilter = function analyzeNeo4jTypeFilter(_ref33) {
  var filterOperationType = _ref33.filterOperationType,
      filterValue = _ref33.filterValue,
      parentFieldName = _ref33.parentFieldName;
  var serializedFilterParam = serializeNeo4jTypeParam({
    filterValue: filterValue,
    filterOperationType: filterOperationType,
    parentFieldName: parentFieldName
  });
  var filterMapValue = {};
  var isListFilterArgument = filterOperationType === 'in' || filterOperationType === 'not_in';

  if (isListFilterArgument) {
    filterMapValue = filterValue.reduce(function (booleanMap, filter) {
      (0, _keys["default"])(filter).forEach(function (key) {
        booleanMap[key] = true;
      });
      return booleanMap;
    }, {});
  } else {
    filterMapValue = (0, _keys["default"])(filterValue).reduce(function (booleanMap, key) {
      booleanMap[key] = true;
      return booleanMap;
    }, {});
  }

  return [serializedFilterParam, filterMapValue];
};

var analyzeNestedFilterArgument = function analyzeNestedFilterArgument(_ref34) {
  var parentSchemaType = _ref34.parentSchemaType,
      parentFieldName = _ref34.parentFieldName,
      schemaType = _ref34.schemaType,
      variableName = _ref34.variableName,
      filterValue = _ref34.filterValue,
      filterParam = _ref34.filterParam,
      schema = _ref34.schema;
  var isList = (0, _isArray["default"])(filterValue); // coersion to array for dynamic iteration of objects and arrays

  if (!isList) filterValue = [filterValue];
  var serializedFilterValue = [];
  var filterValueFieldMap = {};
  filterValue.forEach(function (filter) {
    var serializedValues = {};
    var serializedValue = {};
    var valueFieldMap = {};
    (0, _entries["default"])(filter).forEach(function (_ref35) {
      var _ref36 = (0, _slicedToArray2["default"])(_ref35, 2),
          fieldName = _ref36[0],
          value = _ref36[1];

      fieldName = deserializeFilterFieldName(fieldName);

      var _analyzeFilterArgumen5 = analyzeFilterArgument({
        parentFieldName: parentFieldName,
        filterValue: value,
        filterValues: filter,
        fieldName: fieldName,
        variableName: variableName,
        filterParam: filterParam,
        parentSchemaType: parentSchemaType,
        schemaType: schemaType,
        schema: schema
      });

      var _analyzeFilterArgumen6 = (0, _slicedToArray2["default"])(_analyzeFilterArgumen5, 2);

      serializedValue = _analyzeFilterArgumen6[0];
      valueFieldMap = _analyzeFilterArgumen6[1];
      var filterParamName = serializeFilterFieldName(fieldName, value);
      var filterMapEntry = filterValueFieldMap[filterParamName];
      if (!filterMapEntry) filterValueFieldMap[filterParamName] = valueFieldMap; // deep merges in order to capture differences in objects within nested array filters
      else filterValueFieldMap[filterParamName] = _lodash["default"].merge(filterMapEntry, valueFieldMap);
      serializedValues[filterParamName] = serializedValue;
    });
    serializedFilterValue.push(serializedValues);
  }); // undo array coersion

  if (!isList) serializedFilterValue = serializedFilterValue[0];
  return [serializedFilterValue, filterValueFieldMap];
};

var serializeFilterFieldName = function serializeFilterFieldName(name, value) {
  if (value === null) {
    var parsedFilterName = parseFilterArgumentName(name);
    var filterOperationType = parsedFilterName.type;

    if (!filterOperationType || filterOperationType === 'not') {
      return "_".concat(name, "_null");
    }
  }

  return name;
};

var serializeNeo4jTypeParam = function serializeNeo4jTypeParam(_ref37) {
  var filterValue = _ref37.filterValue,
      filterOperationType = _ref37.filterOperationType,
      parentFieldName = _ref37.parentFieldName;
  var isList = (0, _isArray["default"])(filterValue);
  if (!isList) filterValue = [filterValue];
  var serializedValues = filterValue.reduce(function (serializedValues, filter) {
    var serializedValue = {};

    if (filter['formatted'] && parentFieldName !== 'OR' && parentFieldName !== 'AND' && filterOperationType !== 'in' && filterOperationType !== 'not_in') {
      serializedValue = filter['formatted'];
    } else {
      serializedValue = (0, _entries["default"])(filter).reduce(function (serialized, _ref38) {
        var _ref39 = (0, _slicedToArray2["default"])(_ref38, 2),
            key = _ref39[0],
            value = _ref39[1];

        if ((0, _isInteger["default"])(value)) value = _neo4jDriver["default"]["int"](value);
        serialized[key] = value;
        return serialized;
      }, {});
    }

    serializedValues.push(serializedValue);
    return serializedValues;
  }, []);
  if (!isList) serializedValues = serializedValues[0];
  return serializedValues;
};

var deserializeFilterFieldName = function deserializeFilterFieldName(name) {
  if (name.startsWith('_') && name.endsWith('_null')) {
    name = name.substring(1, name.length - 5);
  }

  return name;
};

var translateFilterArguments = function translateFilterArguments(_ref40) {
  var filterFieldMap = _ref40.filterFieldMap,
      filterCypherParam = _ref40.filterCypherParam,
      variableName = _ref40.variableName,
      rootIsRelationType = _ref40.rootIsRelationType,
      schemaType = _ref40.schemaType,
      parentSchemaType = _ref40.parentSchemaType,
      schema = _ref40.schema;
  return (0, _entries["default"])(filterFieldMap).reduce(function (translations, _ref41) {
    var _ref42 = (0, _slicedToArray2["default"])(_ref41, 2),
        name = _ref42[0],
        value = _ref42[1];

    // the filter field map uses serialized field names to allow for both field: {} and field: null
    name = deserializeFilterFieldName(name);
    var translation = translateFilterArgument({
      filterParam: filterCypherParam,
      fieldName: name,
      filterValue: value,
      rootIsRelationType: rootIsRelationType,
      variableName: variableName,
      schemaType: schemaType,
      parentSchemaType: parentSchemaType,
      schema: schema
    });

    if (translation) {
      translations.push("(".concat(translation, ")"));
    }

    return translations;
  }, []);
};

var translateFilterArgument = function translateFilterArgument(_ref43) {
  var parentParamPath = _ref43.parentParamPath,
      parentFieldName = _ref43.parentFieldName,
      isListFilterArgument = _ref43.isListFilterArgument,
      filterValue = _ref43.filterValue,
      fieldName = _ref43.fieldName,
      rootIsRelationType = _ref43.rootIsRelationType,
      variableName = _ref43.variableName,
      filterParam = _ref43.filterParam,
      schemaType = _ref43.schemaType,
      parentSchemaType = _ref43.parentSchemaType,
      schema = _ref43.schema;
  // parse field name into prefix (ex: name, company) and
  // possible suffix identifying operation type (ex: _gt, _in)
  var parsedFilterName = parseFilterArgumentName(fieldName);
  var filterOperationField = parsedFilterName.name;
  var filterOperationType = parsedFilterName.type;
  var innerSchemaType = schemaType;
  var typeName = schemaType.name;

  if (filterOperationField !== 'OR' && filterOperationField !== 'AND') {
    var schemaTypeFields = schemaType.getFields();
    var filterField = schemaTypeFields[filterOperationField];
    var filterFieldAst = filterField.astNode;
    var filterType = filterFieldAst.type;
    var innerFieldType = (0, _fields.unwrapNamedType)({
      type: filterType
    });
    typeName = innerFieldType.name;
    innerSchemaType = schema.getType(typeName);
  } // build path for parameter data for current filter field


  var parameterPath = "".concat(parentParamPath ? parentParamPath : filterParam, ".").concat(fieldName); // short-circuit evaluation: predicate used to skip a field
  // if processing a list of objects that possibly contain different arguments

  var nullFieldPredicate = decideNullSkippingPredicate({
    parameterPath: parameterPath,
    isListFilterArgument: isListFilterArgument,
    parentParamPath: parentParamPath
  });
  var translation = '';

  if ((0, _graphql.isScalarType)(innerSchemaType) || (0, _graphql.isEnumType)(innerSchemaType)) {
    translation = translateScalarFilter({
      isListFilterArgument: isListFilterArgument,
      filterOperationField: filterOperationField,
      filterOperationType: filterOperationType,
      filterValue: filterValue,
      fieldName: fieldName,
      variableName: variableName,
      parameterPath: parameterPath,
      parentParamPath: parentParamPath,
      filterParam: filterParam,
      nullFieldPredicate: nullFieldPredicate
    });
  } else if ((0, _graphql.isObjectType)(innerSchemaType) || (0, _graphql.isInterfaceType)(innerSchemaType)) {
    translation = translateInputFilter({
      rootIsRelationType: rootIsRelationType,
      isListFilterArgument: isListFilterArgument,
      filterOperationField: filterOperationField,
      filterOperationType: filterOperationType,
      filterValue: filterValue,
      variableName: variableName,
      fieldName: fieldName,
      filterParam: filterParam,
      schema: schema,
      parentSchemaType: parentSchemaType,
      schemaType: schemaType,
      parameterPath: parameterPath,
      parentParamPath: parentParamPath,
      parentFieldName: parentFieldName,
      nullFieldPredicate: nullFieldPredicate
    });
  }

  return translation;
};

var parseFilterArgumentName = function parseFilterArgumentName(fieldName) {
  var fieldNameParts = fieldName.split('_');
  var filterTypes = ['_not', '_in', '_not_in', '_contains', '_not_contains', '_starts_with', '_not_starts_with', '_ends_with', '_not_ends_with', '_lt', '_lte', '_gt', '_gte', '_some', '_none', '_single', '_every', '_distance', '_distance_lt', '_distance_lte', '_distance_gt', '_distance_gte'];
  var filterType = '';

  if (fieldNameParts.length > 1) {
    var regExp = [];

    _lodash["default"].each(filterTypes, function (f) {
      regExp.push(f + '$');
    });

    var regExpJoin = '(' + regExp.join('|') + ')';

    var preparedFieldAndFilterField = _lodash["default"].replace(fieldName, new RegExp(regExpJoin), '[::filterFieldSeperator::]$1');

    var _preparedFieldAndFilt = preparedFieldAndFilterField.split('[::filterFieldSeperator::]'),
        _preparedFieldAndFilt2 = (0, _slicedToArray2["default"])(_preparedFieldAndFilt, 2),
        parsedField = _preparedFieldAndFilt2[0],
        parsedFilterField = _preparedFieldAndFilt2[1];

    fieldName = !_lodash["default"].isUndefined(parsedField) ? parsedField : fieldName;
    filterType = !_lodash["default"].isUndefined(parsedFilterField) ? parsedFilterField.substr(1) : ''; // Strip off first underscore
  }

  return {
    name: fieldName,
    type: filterType
  };
};

var translateScalarFilter = function translateScalarFilter(_ref44) {
  var isListFilterArgument = _ref44.isListFilterArgument,
      filterOperationField = _ref44.filterOperationField,
      filterOperationType = _ref44.filterOperationType,
      filterValue = _ref44.filterValue,
      variableName = _ref44.variableName,
      parameterPath = _ref44.parameterPath,
      parentParamPath = _ref44.parentParamPath,
      filterParam = _ref44.filterParam,
      nullFieldPredicate = _ref44.nullFieldPredicate;
  // build path to node/relationship property
  var propertyPath = "".concat((0, _utils.safeVar)(variableName), ".").concat(filterOperationField);

  if (isExistentialFilter(filterOperationType, filterValue)) {
    return translateNullFilter({
      filterOperationField: filterOperationField,
      filterOperationType: filterOperationType,
      propertyPath: propertyPath,
      filterParam: filterParam,
      parentParamPath: parentParamPath,
      isListFilterArgument: isListFilterArgument
    });
  }

  return "".concat(nullFieldPredicate).concat(buildOperatorExpression({
    filterOperationType: filterOperationType,
    propertyPath: propertyPath
  }), " ").concat(parameterPath);
};

var isExistentialFilter = function isExistentialFilter(type, value) {
  return (!type || type === 'not') && value === null;
};

var decideNullSkippingPredicate = function decideNullSkippingPredicate(_ref45) {
  var parameterPath = _ref45.parameterPath,
      isListFilterArgument = _ref45.isListFilterArgument,
      parentParamPath = _ref45.parentParamPath;
  return isListFilterArgument && parentParamPath ? "".concat(parameterPath, " IS NULL OR ") : '';
};

var translateNullFilter = function translateNullFilter(_ref46) {
  var filterOperationField = _ref46.filterOperationField,
      filterOperationType = _ref46.filterOperationType,
      filterParam = _ref46.filterParam,
      propertyPath = _ref46.propertyPath,
      parentParamPath = _ref46.parentParamPath,
      isListFilterArgument = _ref46.isListFilterArgument;
  var isNegationFilter = filterOperationType === 'not'; // allign with modified parameter names for null filters

  var paramPath = "".concat(parentParamPath ? parentParamPath : filterParam, "._").concat(filterOperationField, "_").concat(isNegationFilter ? "not_" : '', "null"); // build a predicate for checking the existence of a
  // property or relationship

  var predicate = "".concat(paramPath, " = TRUE AND").concat(isNegationFilter ? '' : ' NOT', " EXISTS(").concat(propertyPath, ")"); // skip the field if it is null in the case of it
  // existing within one of many objects in a list filter

  var nullFieldPredicate = decideNullSkippingPredicate({
    parameterPath: paramPath,
    isListFilterArgument: isListFilterArgument,
    parentParamPath: parentParamPath
  });
  return "".concat(nullFieldPredicate).concat(predicate);
};

var buildOperatorExpression = function buildOperatorExpression(_ref47) {
  var filterOperationType = _ref47.filterOperationType,
      propertyPath = _ref47.propertyPath,
      isListFilterArgument = _ref47.isListFilterArgument,
      parameterPath = _ref47.parameterPath;
  if (isListFilterArgument) return "".concat(propertyPath, " =");

  switch (filterOperationType) {
    case 'not':
      return "NOT ".concat(propertyPath, " = ");

    case 'in':
      return "".concat(propertyPath, " IN");

    case 'not_in':
      return "NOT ".concat(propertyPath, " IN");

    case 'contains':
      return "".concat(propertyPath, " CONTAINS");

    case 'not_contains':
      return "NOT ".concat(propertyPath, " CONTAINS");

    case 'starts_with':
      return "".concat(propertyPath, " STARTS WITH");

    case 'not_starts_with':
      return "NOT ".concat(propertyPath, " STARTS WITH");

    case 'ends_with':
      return "".concat(propertyPath, " ENDS WITH");

    case 'not_ends_with':
      return "NOT ".concat(propertyPath, " ENDS WITH");

    case 'distance':
      return "distance(".concat(propertyPath, ", point(").concat(parameterPath, ".point)) =");

    case 'lt':
      return "".concat(propertyPath, " <");

    case 'distance_lt':
      return "distance(".concat(propertyPath, ", point(").concat(parameterPath, ".point)) <");

    case 'lte':
      return "".concat(propertyPath, " <=");

    case 'distance_lte':
      return "distance(".concat(propertyPath, ", point(").concat(parameterPath, ".point)) <=");

    case 'gt':
      return "".concat(propertyPath, " >");

    case 'distance_gt':
      return "distance(".concat(propertyPath, ", point(").concat(parameterPath, ".point)) >");

    case 'gte':
      return "".concat(propertyPath, " >=");

    case 'distance_gte':
      return "distance(".concat(propertyPath, ", point(").concat(parameterPath, ".point)) >=");

    default:
      {
        return "".concat(propertyPath, " =");
      }
  }
};

var translateInputFilter = function translateInputFilter(_ref48) {
  var rootIsRelationType = _ref48.rootIsRelationType,
      isListFilterArgument = _ref48.isListFilterArgument,
      filterOperationField = _ref48.filterOperationField,
      filterOperationType = _ref48.filterOperationType,
      filterValue = _ref48.filterValue,
      variableName = _ref48.variableName,
      fieldName = _ref48.fieldName,
      filterParam = _ref48.filterParam,
      schema = _ref48.schema,
      schemaType = _ref48.schemaType,
      parameterPath = _ref48.parameterPath,
      nullFieldPredicate = _ref48.nullFieldPredicate,
      parentSchemaType = _ref48.parentSchemaType,
      parentParamPath = _ref48.parentParamPath,
      parentFieldName = _ref48.parentFieldName;

  if (fieldName === 'AND' || fieldName === 'OR') {
    return translateLogicalFilter({
      filterValue: filterValue,
      variableName: variableName,
      filterOperationType: filterOperationType,
      filterOperationField: filterOperationField,
      fieldName: fieldName,
      filterParam: filterParam,
      schema: schema,
      schemaType: schemaType,
      parameterPath: parameterPath,
      nullFieldPredicate: nullFieldPredicate
    });
  } else {
    var schemaTypeField = schemaType.getFields()[filterOperationField];
    var innerSchemaType = (0, _utils.innerType)(schemaTypeField.type);
    var typeName = innerSchemaType.name;
    var isObjectTypeFilter = (0, _graphql.isObjectType)(innerSchemaType);
    var isInterfaceTypeFilter = (0, _graphql.isInterfaceType)(innerSchemaType);

    if (isObjectTypeFilter || isInterfaceTypeFilter) {
      var _decideRelationFilter3 = decideRelationFilterMetadata({
        fieldName: fieldName,
        parentSchemaType: parentSchemaType,
        schemaType: schemaType,
        variableName: variableName,
        innerSchemaType: innerSchemaType,
        filterOperationField: filterOperationField
      }),
          _decideRelationFilter4 = (0, _slicedToArray2["default"])(_decideRelationFilter3, 9),
          thisType = _decideRelationFilter4[0],
          relatedType = _decideRelationFilter4[1],
          relationLabel = _decideRelationFilter4[2],
          relationDirection = _decideRelationFilter4[3],
          isRelation = _decideRelationFilter4[4],
          isRelationType = _decideRelationFilter4[5],
          isRelationTypeNode = _decideRelationFilter4[6],
          isReflexiveRelationType = _decideRelationFilter4[7],
          isReflexiveTypeDirectedField = _decideRelationFilter4[8];

      if ((0, _utils.isTemporalType)(typeName) || (0, _utils.isSpatialType)(typeName) || (0, _utils.isSpatialDistanceInputType)({
        filterOperationType: filterOperationType
      })) {
        return translateNeo4jTypeFilter({
          typeName: typeName,
          isRelationTypeNode: isRelationTypeNode,
          filterValue: filterValue,
          variableName: variableName,
          filterOperationField: filterOperationField,
          filterOperationType: filterOperationType,
          fieldName: fieldName,
          filterParam: filterParam,
          parameterPath: parameterPath,
          parentParamPath: parentParamPath,
          isListFilterArgument: isListFilterArgument,
          nullFieldPredicate: nullFieldPredicate
        });
      } else if (isRelation || isRelationType || isRelationTypeNode) {
        return translateRelationFilter({
          rootIsRelationType: rootIsRelationType,
          thisType: thisType,
          relatedType: relatedType,
          relationLabel: relationLabel,
          relationDirection: relationDirection,
          isRelationType: isRelationType,
          isRelationTypeNode: isRelationTypeNode,
          isReflexiveRelationType: isReflexiveRelationType,
          isReflexiveTypeDirectedField: isReflexiveTypeDirectedField,
          filterValue: filterValue,
          variableName: variableName,
          filterOperationField: filterOperationField,
          filterOperationType: filterOperationType,
          fieldName: fieldName,
          filterParam: filterParam,
          schema: schema,
          schemaType: schemaType,
          innerSchemaType: innerSchemaType,
          parameterPath: parameterPath,
          parentParamPath: parentParamPath,
          isListFilterArgument: isListFilterArgument,
          nullFieldPredicate: nullFieldPredicate,
          parentSchemaType: parentSchemaType,
          parentFieldName: parentFieldName
        });
      }
    }
  }
};

var translateLogicalFilter = function translateLogicalFilter(_ref49) {
  var filterValue = _ref49.filterValue,
      variableName = _ref49.variableName,
      filterOperationType = _ref49.filterOperationType,
      filterOperationField = _ref49.filterOperationField,
      fieldName = _ref49.fieldName,
      filterParam = _ref49.filterParam,
      schema = _ref49.schema,
      schemaType = _ref49.schemaType,
      parameterPath = _ref49.parameterPath,
      nullFieldPredicate = _ref49.nullFieldPredicate;
  var listElementVariable = "_".concat(fieldName); // build predicate expressions for all unique arguments within filterValue
  // isListFilterArgument is true here so that nullFieldPredicate is used

  var predicates = buildFilterPredicates({
    filterOperationType: filterOperationType,
    parentFieldName: fieldName,
    listVariable: listElementVariable,
    parentSchemaType: schemaType,
    isListFilterArgument: true,
    schemaType: schemaType,
    variableName: variableName,
    filterValue: filterValue,
    filterParam: filterParam,
    // typeFields,
    schema: schema
  });
  var predicateListVariable = parameterPath; // decide root predicate function

  var rootPredicateFunction = decidePredicateFunction({
    filterOperationField: filterOperationField
  }); // build root predicate expression

  var translation = buildPredicateFunction({
    nullFieldPredicate: nullFieldPredicate,
    predicateListVariable: predicateListVariable,
    rootPredicateFunction: rootPredicateFunction,
    predicates: predicates,
    listElementVariable: listElementVariable
  });
  return translation;
};

var translateRelationFilter = function translateRelationFilter(_ref50) {
  var rootIsRelationType = _ref50.rootIsRelationType,
      thisType = _ref50.thisType,
      relatedType = _ref50.relatedType,
      relationLabel = _ref50.relationLabel,
      relationDirection = _ref50.relationDirection,
      isRelationType = _ref50.isRelationType,
      isRelationTypeNode = _ref50.isRelationTypeNode,
      isReflexiveRelationType = _ref50.isReflexiveRelationType,
      isReflexiveTypeDirectedField = _ref50.isReflexiveTypeDirectedField,
      filterValue = _ref50.filterValue,
      variableName = _ref50.variableName,
      filterOperationField = _ref50.filterOperationField,
      filterOperationType = _ref50.filterOperationType,
      fieldName = _ref50.fieldName,
      filterParam = _ref50.filterParam,
      schema = _ref50.schema,
      schemaType = _ref50.schemaType,
      innerSchemaType = _ref50.innerSchemaType,
      parameterPath = _ref50.parameterPath,
      parentParamPath = _ref50.parentParamPath,
      isListFilterArgument = _ref50.isListFilterArgument,
      nullFieldPredicate = _ref50.nullFieldPredicate,
      parentSchemaType = _ref50.parentSchemaType,
      parentFieldName = _ref50.parentFieldName;

  if (isReflexiveTypeDirectedField) {
    // when at the 'from' or 'to' fields of a reflexive relation type payload
    // we need to use the name of the parent schema type, ex: 'person' for
    // Person.knows gets used here for reflexive path patterns, rather than
    // the normally set 'person_filter_person' variableName
    variableName = parentSchemaType.name.toLowerCase();
  }

  var pathExistencePredicate = buildRelationExistencePath(variableName, relationLabel, relationDirection, relatedType, isRelationTypeNode);

  if (isExistentialFilter(filterOperationType, filterValue)) {
    return translateNullFilter({
      filterOperationField: filterOperationField,
      filterOperationType: filterOperationType,
      propertyPath: pathExistencePredicate,
      filterParam: filterParam,
      parentParamPath: parentParamPath,
      isListFilterArgument: isListFilterArgument
    });
  }

  var parentFilterOperationField = filterOperationField;
  var parentFilterOperationType = filterOperationType;

  if (isReflexiveTypeDirectedField) {
    // causes the 'from' and 'to' fields on the payload of a reflexive
    // relation type to use the parent field name, ex: 'knows_some'
    // is used for 'from' and 'to' in 'knows_some: { from: {}, to: {} }'
    var parsedFilterName = parseFilterArgumentName(parentFieldName);
    parentFilterOperationField = parsedFilterName.name;
    parentFilterOperationType = parsedFilterName.type;
  } // build a list comprehension containing path pattern for related type


  var predicateListVariable = buildRelatedTypeListComprehension({
    rootIsRelationType: rootIsRelationType,
    variableName: variableName,
    thisType: thisType,
    relatedType: relatedType,
    relationLabel: relationLabel,
    relationDirection: relationDirection,
    isRelationTypeNode: isRelationTypeNode,
    isRelationType: isRelationType
  });
  var rootPredicateFunction = decidePredicateFunction({
    isRelationTypeNode: isRelationTypeNode,
    filterOperationField: parentFilterOperationField,
    filterOperationType: parentFilterOperationType
  });
  return buildRelationPredicate({
    rootIsRelationType: rootIsRelationType,
    parentFieldName: parentFieldName,
    isRelationType: isRelationType,
    isListFilterArgument: isListFilterArgument,
    isReflexiveRelationType: isReflexiveRelationType,
    isReflexiveTypeDirectedField: isReflexiveTypeDirectedField,
    thisType: thisType,
    relatedType: relatedType,
    schemaType: schemaType,
    innerSchemaType: innerSchemaType,
    fieldName: fieldName,
    filterOperationType: filterOperationType,
    filterValue: filterValue,
    filterParam: filterParam,
    schema: schema,
    parameterPath: parameterPath,
    nullFieldPredicate: nullFieldPredicate,
    pathExistencePredicate: pathExistencePredicate,
    predicateListVariable: predicateListVariable,
    rootPredicateFunction: rootPredicateFunction
  });
};

var decideRelationFilterMetadata = function decideRelationFilterMetadata(_ref51) {
  var fieldName = _ref51.fieldName,
      parentSchemaType = _ref51.parentSchemaType,
      schemaType = _ref51.schemaType,
      variableName = _ref51.variableName,
      innerSchemaType = _ref51.innerSchemaType,
      filterOperationField = _ref51.filterOperationField;
  var thisType = '';
  var relatedType = '';
  var isRelation = false;
  var isRelationType = false;
  var isRelationTypeNode = false;
  var isReflexiveRelationType = false;
  var isReflexiveTypeDirectedField = false; // @relation field directive

  var _relationDirective = (0, _utils.relationDirective)(schemaType, filterOperationField),
      relLabel = _relationDirective.name,
      relDirection = _relationDirective.direction; // @relation type directive on node type field


  var innerRelationTypeDirective = (0, _utils.getRelationTypeDirective)(innerSchemaType.astNode); // @relation type directive on this type; node type field on relation type
  // If there is no @relation directive on the schemaType, check the parentSchemaType
  // for the same directive obtained above when the relation type is first seen

  var relationTypeDirective = (0, _utils.getRelationTypeDirective)(schemaType.astNode);

  if (relLabel && relDirection) {
    isRelation = true;
    var typeVariables = (0, _utils.typeIdentifiers)(innerSchemaType);
    thisType = schemaType.name;
    relatedType = typeVariables.typeName;
  } else if (innerRelationTypeDirective) {
    isRelationType = true;
    thisType = innerRelationTypeDirective.from;
    relatedType = innerRelationTypeDirective.to;
    relLabel = innerRelationTypeDirective.name;
    relDirection = 'OUT';

    if (thisType === relatedType) {
      isReflexiveRelationType = true;

      if (fieldName === 'from') {
        isReflexiveTypeDirectedField = true;
        relDirection = 'IN';
      } else if (fieldName === 'to') {
        isReflexiveTypeDirectedField = true;
      }
    } else if (thisType !== relatedType) {
      var filteredType = schemaType && schemaType.name ? schemaType.name : '';

      if (filteredType === relatedType) {
        // then a filter argument for the incoming direction is being used
        // when querying the node type it goes out from
        var temp = thisType;
        thisType = relatedType;
        relatedType = temp;
        relDirection = 'IN';
      }
    }
  } else if (relationTypeDirective) {
    isRelationTypeNode = true;
    thisType = relationTypeDirective.from;
    relatedType = relationTypeDirective.to;
    relLabel = variableName;
    relDirection = 'OUT'; // if not a reflexive relationship type

    if (thisType !== relatedType) {
      // When buildFilterPredicates is used in buildRelationPredicate,
      // parentSchemaType is provided and used here to decide whether
      // to filter the incoming or outgoing node type
      var _filteredType = parentSchemaType && parentSchemaType.name ? parentSchemaType.name : ''; // the connecting node type field on a relationship type filter
      // may be incoming or outgoing; thisType could be .from or .to


      if (_filteredType === relatedType) {
        // then this filter argument is being used on a field of the node type
        // the relationship goes .to, so we need to filter for the node types
        // it comes .from
        var _temp = thisType;
        thisType = relatedType;
        relatedType = _temp;
        relDirection = 'IN';
      }
    }
  }

  return [thisType, relatedType, relLabel, relDirection, isRelation, isRelationType, isRelationTypeNode, isReflexiveRelationType, isReflexiveTypeDirectedField];
};

var buildRelationPredicate = function buildRelationPredicate(_ref52) {
  var rootIsRelationType = _ref52.rootIsRelationType,
      isRelationType = _ref52.isRelationType,
      isReflexiveRelationType = _ref52.isReflexiveRelationType,
      isReflexiveTypeDirectedField = _ref52.isReflexiveTypeDirectedField,
      thisType = _ref52.thisType,
      isListFilterArgument = _ref52.isListFilterArgument,
      relatedType = _ref52.relatedType,
      schemaType = _ref52.schemaType,
      innerSchemaType = _ref52.innerSchemaType,
      fieldName = _ref52.fieldName,
      filterOperationType = _ref52.filterOperationType,
      filterValue = _ref52.filterValue,
      filterParam = _ref52.filterParam,
      schema = _ref52.schema,
      parameterPath = _ref52.parameterPath,
      nullFieldPredicate = _ref52.nullFieldPredicate,
      pathExistencePredicate = _ref52.pathExistencePredicate,
      predicateListVariable = _ref52.predicateListVariable,
      rootPredicateFunction = _ref52.rootPredicateFunction;
  var isRelationList = filterOperationType === 'in' || filterOperationType === 'not_in';
  var relationVariable = buildRelationVariable(thisType, relatedType);
  var variableName = relatedType.toLowerCase();
  var listVariable = parameterPath;

  if (rootIsRelationType || isRelationType) {
    // change the variable to be used in filtering
    // to the appropriate relationship variable
    // ex: project -> person_filter_project
    variableName = relationVariable;
  }

  if (isRelationList) {
    // set the base list comprehension variable
    // to point at each array element instead
    // ex: $filter.company_in -> _company_in
    listVariable = "_".concat(fieldName); // set to list to enable null field
    // skipping for all child filters

    isListFilterArgument = true;
  }

  var predicates = buildFilterPredicates({
    parentFieldName: fieldName,
    parentSchemaType: schemaType,
    schemaType: innerSchemaType,
    variableName: variableName,
    isListFilterArgument: isListFilterArgument,
    listVariable: listVariable,
    filterOperationType: filterOperationType,
    isRelationType: isRelationType,
    filterValue: filterValue,
    filterParam: filterParam,
    schema: schema
  });

  if (isRelationList) {
    predicates = buildPredicateFunction({
      predicateListVariable: parameterPath,
      listElementVariable: listVariable,
      rootPredicateFunction: rootPredicateFunction,
      predicates: predicates
    });
    rootPredicateFunction = decidePredicateFunction({
      isRelationList: isRelationList
    });
  }

  if (isReflexiveRelationType && !isReflexiveTypeDirectedField) {
    // At reflexive relation type fields, sufficient predicates and values are already
    // obtained from the above call to the recursive buildFilterPredicates
    // ex: Person.knows, Person.knows_in, etc.
    // Note: Since only the internal 'from' and 'to' fields are translated for reflexive
    // relation types, their translations will use the fieldName and schema type name
    // of this field. See: the top of translateRelationFilter
    return predicates;
  }

  var listElementVariable = (0, _utils.safeVar)(variableName);
  return buildPredicateFunction({
    nullFieldPredicate: nullFieldPredicate,
    pathExistencePredicate: pathExistencePredicate,
    predicateListVariable: predicateListVariable,
    rootPredicateFunction: rootPredicateFunction,
    predicates: predicates,
    listElementVariable: listElementVariable
  });
};

var buildPredicateFunction = function buildPredicateFunction(_ref53) {
  var nullFieldPredicate = _ref53.nullFieldPredicate,
      pathExistencePredicate = _ref53.pathExistencePredicate,
      predicateListVariable = _ref53.predicateListVariable,
      rootPredicateFunction = _ref53.rootPredicateFunction,
      predicates = _ref53.predicates,
      listElementVariable = _ref53.listElementVariable;
  // https://neo4j.com/docs/cypher-manual/current/functions/predicate/
  return "".concat(nullFieldPredicate || '').concat(pathExistencePredicate ? "EXISTS(".concat(pathExistencePredicate, ") AND ") : '').concat(rootPredicateFunction, "(").concat(listElementVariable, " IN ").concat(predicateListVariable, " WHERE ").concat(predicates, ")");
};

var buildRelationVariable = function buildRelationVariable(thisType, relatedType) {
  return "".concat(thisType.toLowerCase(), "_filter_").concat(relatedType.toLowerCase());
};

var decidePredicateFunction = function decidePredicateFunction(_ref54) {
  var filterOperationField = _ref54.filterOperationField,
      filterOperationType = _ref54.filterOperationType,
      isRelationTypeNode = _ref54.isRelationTypeNode,
      isRelationList = _ref54.isRelationList;
  if (filterOperationField === 'AND') return 'ALL';else if (filterOperationField === 'OR') return 'ANY';else if (isRelationTypeNode) return 'ALL';else if (isRelationList) return 'ALL';else {
    switch (filterOperationType) {
      case 'not':
        return 'NONE';

      case 'in':
        return 'ANY';

      case 'not_in':
        return 'NONE';

      case 'some':
        return 'ANY';

      case 'every':
        return 'ALL';

      case 'none':
        return 'NONE';

      case 'single':
        return 'SINGLE';

      case 'distance':
      case 'distance_lt':
      case 'distance_lte':
      case 'distance_gt':
      case 'distance_gte':
        return 'distance';

      default:
        return 'ALL';
    }
  }
};

var buildRelatedTypeListComprehension = function buildRelatedTypeListComprehension(_ref55) {
  var rootIsRelationType = _ref55.rootIsRelationType,
      variableName = _ref55.variableName,
      thisType = _ref55.thisType,
      relatedType = _ref55.relatedType,
      relationLabel = _ref55.relationLabel,
      relationDirection = _ref55.relationDirection,
      isRelationTypeNode = _ref55.isRelationTypeNode,
      isRelationType = _ref55.isRelationType;
  var relationVariable = buildRelationVariable(thisType, relatedType);

  if (rootIsRelationType) {
    relationVariable = variableName;
  }

  var thisTypeVariable = !rootIsRelationType && !isRelationTypeNode ? (0, _utils.safeVar)((0, _utils.lowFirstLetter)(variableName)) : (0, _utils.safeVar)((0, _utils.lowFirstLetter)(thisType)); // prevents related node variable from
  // conflicting with parent variables

  var relatedTypeVariable = (0, _utils.safeVar)("_".concat(relatedType.toLowerCase())); // builds a path pattern within a list comprehension
  // that extracts related nodes

  return "[(".concat(thisTypeVariable, ")").concat(relationDirection === 'IN' ? '<' : '', "-[").concat(isRelationType ? (0, _utils.safeVar)("_".concat(relationVariable)) : isRelationTypeNode ? (0, _utils.safeVar)(relationVariable) : '').concat(!isRelationTypeNode ? ":".concat(relationLabel) : '', "]-").concat(relationDirection === 'OUT' ? '>' : '', "(").concat(isRelationType ? '' : relatedTypeVariable, ":").concat(relatedType, ") | ").concat(isRelationType ? (0, _utils.safeVar)("_".concat(relationVariable)) : relatedTypeVariable, "]");
};

var buildRelationExistencePath = function buildRelationExistencePath(fromVar, relLabel, relDirection, toType, isRelationTypeNode) {
  // because ALL(n IN [] WHERE n) currently returns true
  // an existence predicate is added to make sure a relationship exists
  // otherwise a node returns when it has 0 such relationships, since the
  // predicate function then evaluates an empty list
  var safeFromVar = (0, _utils.safeVar)(fromVar);
  return !isRelationTypeNode ? "(".concat(safeFromVar, ")").concat(relDirection === 'IN' ? '<' : '', "-[:").concat(relLabel, "]-").concat(relDirection === 'OUT' ? '>' : '', "(:").concat(toType, ")") : '';
};

var buildFilterPredicates = function buildFilterPredicates(_ref56) {
  var parentSchemaType = _ref56.parentSchemaType,
      parentFieldName = _ref56.parentFieldName,
      schemaType = _ref56.schemaType,
      variableName = _ref56.variableName,
      listVariable = _ref56.listVariable,
      filterValue = _ref56.filterValue,
      filterParam = _ref56.filterParam,
      schema = _ref56.schema,
      isListFilterArgument = _ref56.isListFilterArgument;
  return (0, _entries["default"])(filterValue).reduce(function (predicates, _ref57) {
    var _ref58 = (0, _slicedToArray2["default"])(_ref57, 2),
        name = _ref58[0],
        value = _ref58[1];

    name = deserializeFilterFieldName(name);
    var predicate = translateFilterArgument({
      parentParamPath: listVariable,
      fieldName: name,
      filterValue: value,
      parentFieldName: parentFieldName,
      parentSchemaType: parentSchemaType,
      isListFilterArgument: isListFilterArgument,
      variableName: variableName,
      filterParam: filterParam,
      schemaType: schemaType,
      schema: schema
    });

    if (predicate) {
      predicates.push("(".concat(predicate, ")"));
    }

    return predicates;
  }, []).join(' AND ');
};

var decideNeo4jTypeFilter = function decideNeo4jTypeFilter(_ref59) {
  var filterOperationType = _ref59.filterOperationType,
      typeName = _ref59.typeName;
  var cypherTypeConstructor = '';
  var isTemporalFilter = false;
  var isSpatialFilter = false;

  if (!(0, _utils.isSpatialDistanceInputType)({
    filterOperationType: filterOperationType
  })) {
    switch (typeName) {
      case '_Neo4jTime':
        {
          isTemporalFilter = true;
          cypherTypeConstructor = 'time';
          break;
        }

      case '_Neo4jDate':
        {
          isTemporalFilter = true;
          cypherTypeConstructor = 'date';
          break;
        }

      case '_Neo4jDateTime':
        {
          isTemporalFilter = true;
          cypherTypeConstructor = 'datetime';
          break;
        }

      case '_Neo4jLocalTime':
        {
          isTemporalFilter = true;
          cypherTypeConstructor = 'localtime';
          break;
        }

      case '_Neo4jLocalDateTime':
        {
          isTemporalFilter = true;
          cypherTypeConstructor = 'localdatetime';
          break;
        }

      case '_Neo4jPoint':
        {
          isSpatialFilter = true;
          cypherTypeConstructor = 'point';
          break;
        }
    }
  }

  return [isTemporalFilter, isSpatialFilter, cypherTypeConstructor];
};

var translateNeo4jTypeFilter = function translateNeo4jTypeFilter(_ref60) {
  var typeName = _ref60.typeName,
      isRelationTypeNode = _ref60.isRelationTypeNode,
      filterValue = _ref60.filterValue,
      variableName = _ref60.variableName,
      filterOperationField = _ref60.filterOperationField,
      filterOperationType = _ref60.filterOperationType,
      fieldName = _ref60.fieldName,
      filterParam = _ref60.filterParam,
      parameterPath = _ref60.parameterPath,
      parentParamPath = _ref60.parentParamPath,
      isListFilterArgument = _ref60.isListFilterArgument,
      nullFieldPredicate = _ref60.nullFieldPredicate;
  var safeVariableName = (0, _utils.safeVar)(variableName);
  var propertyPath = "".concat(safeVariableName, ".").concat(filterOperationField);
  var predicate = '';

  var _decideNeo4jTypeFilte = decideNeo4jTypeFilter({
    filterOperationType: filterOperationType,
    typeName: typeName
  }),
      _decideNeo4jTypeFilte2 = (0, _slicedToArray2["default"])(_decideNeo4jTypeFilte, 3),
      isTemporalFilter = _decideNeo4jTypeFilte2[0],
      isSpatialFilter = _decideNeo4jTypeFilte2[1],
      cypherTypeConstructor = _decideNeo4jTypeFilte2[2];

  if (isExistentialFilter(filterOperationType, filterValue)) {
    return translateNullFilter({
      filterOperationField: filterOperationField,
      filterOperationType: filterOperationType,
      propertyPath: propertyPath,
      filterParam: filterParam,
      parentParamPath: parentParamPath,
      isListFilterArgument: isListFilterArgument
    });
  }

  var rootPredicateFunction = decidePredicateFunction({
    isRelationTypeNode: isRelationTypeNode,
    filterOperationField: filterOperationField,
    filterOperationType: filterOperationType
  });
  predicate = buildNeo4jTypePredicate({
    fieldName: fieldName,
    filterOperationField: filterOperationField,
    filterOperationType: filterOperationType,
    filterValue: filterValue,
    parameterPath: parameterPath,
    variableName: variableName,
    nullFieldPredicate: nullFieldPredicate,
    rootPredicateFunction: rootPredicateFunction,
    cypherTypeConstructor: cypherTypeConstructor,
    isTemporalFilter: isTemporalFilter,
    isSpatialFilter: isSpatialFilter
  });
  return predicate;
};

var buildNeo4jTypeTranslation = function buildNeo4jTypeTranslation(_ref61) {
  var filterOperationType = _ref61.filterOperationType,
      listVariable = _ref61.listVariable,
      isTemporalFilter = _ref61.isTemporalFilter,
      isSpatialFilter = _ref61.isSpatialFilter,
      isListFilterArgument = _ref61.isListFilterArgument,
      filterValue = _ref61.filterValue,
      nullFieldPredicate = _ref61.nullFieldPredicate,
      propertyPath = _ref61.propertyPath,
      cypherTypeConstructor = _ref61.cypherTypeConstructor,
      operatorExpression = _ref61.operatorExpression,
      parameterPath = _ref61.parameterPath,
      rootPredicateFunction = _ref61.rootPredicateFunction;

  if ((0, _utils.isSpatialDistanceInputType)({
    filterOperationType: filterOperationType
  })) {
    listVariable = "".concat(listVariable, ".distance");
  }

  var translation = '';
  var isIdentityFilter = !filterOperationType || filterOperationType === 'not';

  if ((isTemporalFilter || isSpatialFilter) && (isIdentityFilter || isListFilterArgument)) {
    var generalizedComparisonPredicates = (0, _keys["default"])(filterValue).map(function (filterName) {
      var isTemporalFormatted = isTemporalFilter && filterName === 'formatted';

      if (nullFieldPredicate || isListFilterArgument) {
        nullFieldPredicate = "".concat(listVariable, ".").concat(filterName, " IS NULL OR ");
      }

      if (isTemporalFormatted) {
        return "(".concat(nullFieldPredicate).concat(propertyPath, " = ").concat(cypherTypeConstructor, "(").concat(listVariable, ".").concat(filterName, "))");
      } else {
        return "(".concat(nullFieldPredicate).concat(propertyPath, ".").concat(filterName, " = ").concat(listVariable, ".").concat(filterName, ")");
      }
    });
    translation = "(".concat(generalizedComparisonPredicates.join(' AND '), ")");

    if (filterOperationType === 'not') {
      translation = "NOT".concat(translation);
    }
  } else {
    translation = "(".concat(nullFieldPredicate).concat(operatorExpression, " ").concat(cypherTypeConstructor, "(").concat(listVariable, "))");
  }

  if (isListFilterArgument) {
    translation = buildPredicateFunction({
      predicateListVariable: parameterPath,
      listElementVariable: listVariable,
      rootPredicateFunction: rootPredicateFunction,
      predicates: translation
    });
  }

  return translation;
};

var buildNeo4jTypePredicate = function buildNeo4jTypePredicate(_ref62) {
  var fieldName = _ref62.fieldName,
      filterOperationField = _ref62.filterOperationField,
      filterOperationType = _ref62.filterOperationType,
      filterValue = _ref62.filterValue,
      parameterPath = _ref62.parameterPath,
      variableName = _ref62.variableName,
      nullFieldPredicate = _ref62.nullFieldPredicate,
      rootPredicateFunction = _ref62.rootPredicateFunction,
      cypherTypeConstructor = _ref62.cypherTypeConstructor,
      isTemporalFilter = _ref62.isTemporalFilter,
      isSpatialFilter = _ref62.isSpatialFilter;
  var isListFilterArgument = filterOperationType === 'in' || filterOperationType === 'not_in'; // ex: project -> person_filter_project

  var listVariable = parameterPath; // ex: $filter.datetime_in -> _datetime_in

  if (isListFilterArgument) listVariable = "_".concat(fieldName);
  var safeVariableName = (0, _utils.safeVar)(variableName);
  var propertyPath = "".concat(safeVariableName, ".").concat(filterOperationField);
  var operatorExpression = buildOperatorExpression({
    filterOperationType: filterOperationType,
    propertyPath: propertyPath,
    isListFilterArgument: isListFilterArgument,
    parameterPath: parameterPath
  });
  var translation = buildNeo4jTypeTranslation({
    filterOperationType: filterOperationType,
    listVariable: listVariable,
    isTemporalFilter: isTemporalFilter,
    isSpatialFilter: isSpatialFilter,
    isListFilterArgument: isListFilterArgument,
    filterValue: filterValue,
    nullFieldPredicate: nullFieldPredicate,
    propertyPath: propertyPath,
    cypherTypeConstructor: cypherTypeConstructor,
    operatorExpression: operatorExpression,
    parameterPath: parameterPath,
    rootPredicateFunction: rootPredicateFunction
  });
  return translation;
};