const { ApolloServer } = require("apollo-server");
const neo4j = require("neo4j-driver").v1;
const { makeAugmentedSchema } = require("neo4j-graphql-js");
const dotenv = require("dotenv");

// set environment variables from ../.env
dotenv.config();



/*
 * Create an executable GraphQL schema object from GraphQL type definitions
 * including autogenerated queries and mutations.
 * Optionally a config object can be included to specify which types to include
 * in generated queries and/or mutations. Read more in the docs:
 * https://grandstack.io/docs/neo4j-graphql-js-api.html#makeaugmentedschemaoptions-graphqlschema
 */

const schema = makeAugmentedSchema({
  typeDefs: process.argv[5] || process.env.GRAPHQL_TYPEDEFS
});

console.log(process.argv[0]);
console.log(process.argv[1]);
console.log(process.argv[2]);
/*
 * Create a Neo4j driver instance to connect to the database
 * using credentials specified as environment variables
 * with fallback to defaults
 */
const driver = neo4j.driver(
  process.argv[2] || process.env.NEO4J_URI,
  neo4j.auth.basic(
    process.argv[3] || process.env.NEO4J_USER,
    process.argv[4] || process.env.NEO4J_PASSWORD
  )
);


/*
 * Create a new ApolloServer instance, serving the GraphQL schema
 * created using makeAugmentedSchema above and injecting the Neo4j driver
 * instance into the context object so it is available in the
 * generated resolvers to connect to the database.
 */
const server = new ApolloServer({
  context: { driver },
  schema: schema,
  introspection: true,
  playground: true
});

// Specify port and path for GraphQL endpoint
const port = process.env.GRAPHQL_LISTEN_PORT || 4001;

server.listen({port}, () => {
  console.log(`GraphQL server ready at http://localhost:${port}`);
});
